@@ -1,25 +1,16 @@
 /*
- * "$Id: ieee1284.c 5591 2006-05-26 19:51:59Z mike $"
+ * "$Id: ieee1284.c 6649 2007-07-11 21:46:42Z mike $"
  *
  *   IEEE-1284 support functions for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2006 by Easy Software Products, all rights reserved.
+ *   Copyright 2007 by Apple Inc.
+ *   Copyright 1997-2007 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
  *   "LICENSE" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
  *   This file is subject to the Apple OS-Developed Software exception.
  *
@@ -93,84 +84,87 @@ backendGetDeviceID(
   * Range check input...
   */
 
-  if (fd < 0 ||
-      !device_id || device_id_size < 32 ||
-      !make_model || make_model_size < 32)
+  if (!device_id || device_id_size < 32)
   {
     DEBUG_puts("backendGetDeviceID: Bad args!");
     return (-1);
   }
 
-  *device_id  = '\0';
-  *make_model = '\0';
+  if (make_model)
+    *make_model = '\0';
 
   if (uri)
     *uri = '\0';
 
- /*
-  * Get the device ID string...
-  */
-
-#ifdef __linux
-  if (!ioctl(fd, LPIOC_GET_DEVICE_ID(device_id_size), device_id))
+  if (fd >= 0)
   {
    /*
-    * Extract the length of the device ID string from the first two
-    * bytes.  The 1284 spec says the length is stored MSB first...
+    * Get the device ID string...
     */
 
-    length = (((unsigned)device_id[0] & 255) << 8) +
-	     ((unsigned)device_id[1] & 255);
+    *device_id = '\0';
 
-   /*
-    * Check to see if the length is larger than our buffer; first
-    * assume that the vendor incorrectly implemented the 1284 spec,
-    * and then limit the length to the size of our buffer...
-    */
+#ifdef __linux
+    if (!ioctl(fd, LPIOC_GET_DEVICE_ID(device_id_size), device_id))
+    {
+     /*
+      * Extract the length of the device ID string from the first two
+      * bytes.  The 1284 spec says the length is stored MSB first...
+      */
 
-    if (length > (device_id_size - 2))
-      length = (((unsigned)device_id[1] & 255) << 8) +
-	       ((unsigned)device_id[0] & 255);
+      length = (((unsigned)device_id[0] & 255) << 8) +
+	       ((unsigned)device_id[1] & 255);
 
-    if (length > (device_id_size - 2))
-      length = device_id_size - 2;
+     /*
+      * Check to see if the length is larger than our buffer; first
+      * assume that the vendor incorrectly implemented the 1284 spec,
+      * and then limit the length to the size of our buffer...
+      */
 
-   /*
-    * Copy the device ID text to the beginning of the buffer and
-    * nul-terminate.
-    */
+      if (length > (device_id_size - 2))
+	length = (((unsigned)device_id[1] & 255) << 8) +
+		 ((unsigned)device_id[0] & 255);
 
-    memmove(device_id, device_id + 2, length);
-    device_id[length] = '\0';
-  }
+      if (length > (device_id_size - 2))
+	length = device_id_size - 2;
+
+     /*
+      * Copy the device ID text to the beginning of the buffer and
+      * nul-terminate.
+      */
+
+      memmove(device_id, device_id + 2, length);
+      device_id[length] = '\0';
+    }
 #  ifdef DEBUG
-  else
-    printf("backendGetDeviceID: ioctl failed - %s\n", strerror(errno));
+    else
+      printf("backendGetDeviceID: ioctl failed - %s\n", strerror(errno));
 #  endif /* DEBUG */
 #endif /* __linux */
 
 #if defined(__sun) && defined(ECPPIOC_GETDEVID)
-  did.mode = ECPP_CENTRONICS;
-  did.len  = device_id_size - 1;
-  did.rlen = 0;
-  did.addr = device_id;
+    did.mode = ECPP_CENTRONICS;
+    did.len  = device_id_size - 1;
+    did.rlen = 0;
+    did.addr = device_id;
 
-  if (!ioctl(fd, ECPPIOC_GETDEVID, &did))
-  {
-   /*
-    * Nul-terminate the device ID text.
-    */
+    if (!ioctl(fd, ECPPIOC_GETDEVID, &did))
+    {
+     /*
+      * Nul-terminate the device ID text.
+      */
 
-    if (did.rlen < (device_id_size - 1))
-      device_id[did.rlen] = '\0';
-    else
-      device_id[device_id_size - 1] = '\0';
-  }
+      if (did.rlen < (device_id_size - 1))
+	device_id[did.rlen] = '\0';
+      else
+	device_id[device_id_size - 1] = '\0';
+    }
 #  ifdef DEBUG
-  else
-    printf("backendGetDeviceID: ioctl failed - %s\n", strerror(errno));
+    else
+      printf("backendGetDeviceID: ioctl failed - %s\n", strerror(errno));
 #  endif /* DEBUG */
 #endif /* __sun && ECPPIOC_GETDEVID */
+  }
 
   DEBUG_printf(("backendGetDeviceID: device_id=\"%s\"\n", device_id));
 
@@ -181,7 +175,8 @@ backendGetDeviceID(
   * Get the make and model...
   */
 
-  backendGetMakeModel(device_id, make_model, make_model_size);
+  if (make_model)
+    backendGetMakeModel(device_id, make_model, make_model_size);
 
  /*
   * Then generate a device URI...
@@ -233,6 +228,8 @@ backendGetDeviceID(
 
       if (!strcasecmp(manufacturer, "Hewlett-Packard"))
         strcpy(manufacturer, "HP");
+      else if (!strcasecmp(manufacturer, "Lexmark International"))
+        strcpy(manufacturer, "Lexmark");
     }
     else
     {
@@ -399,6 +396,8 @@ backendGetMakeModel(
     {
       if (!strncasecmp(mfg, "Hewlett-Packard", 15))
 	strlcpy(make_model, "HP", make_model_size);
+      else if (!strncasecmp(mfg, "Lexmark International", 21))
+	strlcpy(make_model, "Lexmark", make_model_size);
       else
 	strlcpy(make_model, mfg, make_model_size);
 
@@ -495,5 +494,5 @@ backendGetMakeModel(
 
 
 /*
- * End of "$Id: ieee1284.c 5591 2006-05-26 19:51:59Z mike $".
+ * End of "$Id: ieee1284.c 6649 2007-07-11 21:46:42Z mike $".
  */
