@@ -1,25 +1,16 @@
 /*
- * "$Id: ipp.c 6214 2007-01-23 17:01:48Z mike $"
+ * "$Id: ipp.c 6967 2007-09-17 23:30:52Z mike $"
  *
  *   IPP backend for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2006 by Easy Software Products, all rights reserved.
+ *   Copyright 2007 by Apple Inc.
+ *   Copyright 1997-2007 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
  *   "LICENSE" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
  *   This file is subject to the Apple OS-Developed Software exception.
  *
@@ -50,16 +41,17 @@
 #include <cups/backend.h>
 #include <cups/cups.h>
 #include <cups/language.h>
+#include <cups/i18n.h>
 #include <cups/string.h>
 #include <signal.h>
 #include <sys/wait.h>
 
-
 /*
  * Globals...
  */
 
 static char	*password = NULL;	/* Password for device URI */
+static int	password_tries = 0;	/* Password tries */
 #ifdef __APPLE__
 static char	pstmpname[1024] = "";	/* Temporary PostScript file name */
 #endif /* __APPLE__ */
@@ -75,12 +67,12 @@ static void	cancel_job(http_t *http, const char *uri, int id,
 		           const char *resource, const char *user, int version);
 static void	check_printer_state(http_t *http, const char *uri,
 		                    const char *resource, const char *user,
-				    int version);
+				    int version, int job_id);
 #ifdef HAVE_LIBZ
 static void	compress_files(int num_files, char **files);
 #endif /* HAVE_LIBZ */
 static const char *password_cb(const char *);
-static int	report_printer_state(ipp_t *ipp);
+static int	report_printer_state(ipp_t *ipp, int job_id);
 
 #ifdef __APPLE__
 static int	run_pictwps_filter(char **argv, const char *filename);
@@ -101,6 +93,7 @@ main(int  argc,				/* I - Number of command-line args */
      char *argv[])			/* I - Command-line arguments */
 {
   int		i;			/* Looping var */
+  int		send_options;		/* Send job options? */
   int		num_options;		/* Number of printer options */
   cups_option_t	*options;		/* Printer options */
   char		method[255],		/* Method in URI */
@@ -109,9 +102,9 @@ main(int  argc,				/* I - Number of command-line args */
 		resource[1024],		/* Resource info (printer name) */
 		addrname[256],		/* Address name */
 		*optptr,		/* Pointer to URI options */
-		name[255],		/* Name of option */
-		value[255],		/* Value of option */
-		*ptr;			/* Pointer into name or value */
+		*name,			/* Name of option */
+		*value,			/* Value of option */
+		sep;			/* Separator character */
   int		num_files;		/* Number of files to print */
   char		**files,		/* Files to print */
 		*filename;		/* Pointer to single filename */
@@ -122,6 +115,10 @@ main(int  argc,				/* I - Number of command-line args */
   ipp_t		*request,		/* IPP request */
 		*response,		/* IPP response */
 		*supported;		/* get-printer-attributes response */
+  time_t	start_time;		/* Time of first connect */
+  int		recoverable;		/* Recoverable error shown? */
+  int		contimeout;		/* Connection timeout */
+  int		delay;			/* Delay for retries... */
   int		compression,		/* Do compression of the job data? */
 		waitjob,		/* Wait for job complete? */
 		waitprinter;		/* Wait for printer ready? */
@@ -135,7 +132,8 @@ main(int  argc,				/* I - Number of command-line args */
   ipp_attribute_t *printer_accepting;	/* printer-is-accepting-jobs */
   int		copies,			/* Number of copies for job */
 		copies_remaining;	/* Number of copies remaining */
-  const char	*content_type;		/* CONTENT_TYPE environment variable */
+  const char	*content_type,		/* CONTENT_TYPE environment variable */
+		*final_content_type;	/* FINAL_CONTENT_TYPE environment var */
 #if defined(HAVE_SIGACTION) && !defined(HAVE_SIGSET)
   struct sigaction action;		/* Actions for POSIX signals */
 #endif /* HAVE_SIGACTION && !HAVE_SIGSET */
@@ -146,6 +144,7 @@ main(int  argc,				/* I - Number of command-line args */
 		  "document-format-supported",
 		  "printer-is-accepting-jobs",
 		  "printer-state",
+		  "printer-state-message",
 		  "printer-state-reasons",
 		};
   static const char * const jattrs[] =
@@ -201,9 +200,9 @@ main(int  argc,				/* I - Number of command-line args */
   }
   else if (argc < 6)
   {
-    fprintf(stderr,
-            "Usage: %s job-id user title copies options [file ... fileN]\n",
-            argv[0]);
+    _cupsLangPrintf(stderr,
+                    _("Usage: %s job-id user title copies options [file]\n"),
+		    argv[0]);
     return (CUPS_BACKEND_STOP);
   }
 
@@ -211,12 +210,16 @@ main(int  argc,				/* I - Number of command-line args */
   * Get the (final) content type...
   */
 
-  if ((content_type = getenv("FINAL_CONTENT_TYPE")) == NULL)
-    if ((content_type = getenv("CONTENT_TYPE")) == NULL)
-      content_type = "application/octet-stream";
+  if ((content_type = getenv("CONTENT_TYPE")) == NULL)
+    content_type = "application/octet-stream";
 
-  if (!strncmp(content_type, "printer/", 8))
-    content_type = "application/vnd.cups-raw";
+  if ((final_content_type = getenv("FINAL_CONTENT_TYPE")) == NULL)
+  {
+    final_content_type = content_type;
+
+    if (!strncmp(final_content_type, "printer/", 8))
+      final_content_type = "application/vnd.cups-raw";
+  }
 
  /*
   * Extract the hostname and printer name from the URI...
@@ -227,8 +230,9 @@ main(int  argc,				/* I - Number of command-line args */
 		      hostname, sizeof(hostname), &port,
 		      resource, sizeof(resource)) < HTTP_URI_OK)
   {
-    fputs("ERROR: Missing device URI on command-line and no DEVICE_URI "
-          "environment variable!\n", stderr);
+    _cupsLangPuts(stderr,
+                  _("ERROR: Missing device URI on command-line and no "
+		    "DEVICE_URI environment variable!\n"));
     return (CUPS_BACKEND_STOP);
   }
 
@@ -245,6 +249,7 @@ main(int  argc,				/* I - Number of command-line args */
   version     = 1;
   waitjob     = 1;
   waitprinter = 1;
+  contimeout  = 7 * 24 * 60 * 60;
 
   if ((optptr = strchr(resource, '?')) != NULL)
   {
@@ -265,29 +270,30 @@ main(int  argc,				/* I - Number of command-line args */
       * Get the name...
       */
 
-      for (ptr = name; *optptr && *optptr != '=';)
-        if (ptr < (name + sizeof(name) - 1))
-          *ptr++ = *optptr++;
-      *ptr = '\0';
+      name = optptr;
 
-      if (*optptr == '=')
+      while (*optptr && *optptr != '=' && *optptr != '+' && *optptr != '&')
+        optptr ++;
+
+      if ((sep = *optptr) != '\0')
+        *optptr++ = '\0';
+
+      if (sep == '=')
       {
        /*
         * Get the value...
 	*/
 
-        optptr ++;
+        value = optptr;
 
-	for (ptr = value; *optptr && *optptr != '+' && *optptr != '&';)
-          if (ptr < (value + sizeof(value) - 1))
-            *ptr++ = *optptr++;
-	*ptr = '\0';
-
-	if (*optptr == '+' || *optptr == '&')
+	while (*optptr && *optptr != '+' && *optptr != '&')
 	  optptr ++;
+
+        if (*optptr)
+	  *optptr++ = '\0';
       }
       else
-        value[0] = '\0';
+        value = (char *)"";
 
      /*
       * Process the option...
@@ -329,8 +335,9 @@ main(int  argc,				/* I - Number of command-line args */
 	  cupsSetEncryption(HTTP_ENCRYPT_IF_REQUESTED);
 	else
 	{
-	  fprintf(stderr, "ERROR: Unknown encryption option value \"%s\"!\n",
-	          value);
+	  _cupsLangPrintf(stderr,
+	                  _("ERROR: Unknown encryption option value \"%s\"!\n"),
+	        	  value);
         }
       }
       else if (!strcasecmp(name, "version"))
@@ -341,8 +348,9 @@ main(int  argc,				/* I - Number of command-line args */
 	  version = 1;
 	else
 	{
-	  fprintf(stderr, "ERROR: Unknown version option value \"%s\"!\n",
-	          value);
+	  _cupsLangPrintf(stderr,
+	                  _("ERROR: Unknown version option value \"%s\"!\n"),
+	        	  value);
 	}
       }
 #ifdef HAVE_LIBZ
@@ -354,14 +362,24 @@ main(int  argc,				/* I - Number of command-line args */
 	              !strcasecmp(value, "gzip");
       }
 #endif /* HAVE_LIBZ */
+      else if (!strcasecmp(name, "contimeout"))
+      {
+       /*
+        * Set the connection timeout...
+	*/
+
+	if (atoi(value) > 0)
+	  contimeout = atoi(value);
+      }
       else
       {
        /*
         * Unknown option...
 	*/
 
-	fprintf(stderr, "ERROR: Unknown option \"%s\" with value \"%s\"!\n",
-	        name, value);
+	_cupsLangPrintf(stderr,
+	                _("ERROR: Unknown option \"%s\" with value \"%s\"!\n"),
+			name, value);
       }
     }
   }
@@ -413,9 +431,10 @@ main(int  argc,				/* I - Number of command-line args */
     * Point to the single file from stdin...
     */
 
-    filename  = tmpfilename;
-    files     = &filename;
-    num_files = 1;
+    filename     = tmpfilename;
+    num_files    = 1;
+    files        = &filename;
+    send_options = 0;
   }
   else
   {
@@ -423,8 +442,9 @@ main(int  argc,				/* I - Number of command-line args */
     * Point to the files on the command-line...
     */
 
-    num_files = argc - 6;
-    files     = argv + 6;
+    num_files    = argc - 6;
+    files        = argv + 6;
+    send_options = 1;
 
 #ifdef HAVE_LIBZ
     if (compression)
@@ -454,74 +474,37 @@ main(int  argc,				/* I - Number of command-line args */
   else if (!getuid())
   {
    /*
-    * Try loading authentication information from the a##### file.
+    * Try loading authentication information from the environment.
     */
 
-    const char	*request_root;		/* CUPS_REQUESTROOT env var */
-    char	afilename[1024],	/* a##### filename */
-		aline[1024];		/* Line from file */
-    FILE	*fp;			/* File pointer */
+    const char *ptr = getenv("AUTH_USERNAME");
 
+    if (ptr)
+      cupsSetUser(ptr);
 
-    if ((request_root = getenv("CUPS_REQUESTROOT")) != NULL)
-    {
-     /*
-      * Try opening authentication cache file...
-      */
-
-      snprintf(afilename, sizeof(afilename), "%s/a%05d", request_root,
-               atoi(argv[1]));
-      if ((fp = fopen(afilename, "r")) != NULL)
-      {
-       /*
-        * Read username...
-	*/
-
-        if (fgets(aline, sizeof(aline), fp))
-	{
-	 /*
-	  * Decode username...
-	  */
-
-          i = sizeof(username);
-	  httpDecode64_2(username, &i, aline);
-
-         /*
-	  * Read password...
-	  */
-
-	  if (fgets(aline, sizeof(aline), fp))
-	  {
-	   /*
-	    * Decode password...
-	    */
-
-	    i = sizeof(password);
-	    httpDecode64_2(password, &i, aline);
-	  }
-	}
-
-       /*
-        * Close the file...
-	*/
-
-        fclose(fp);
-      }
-    }
+    password = getenv("AUTH_PASSWORD");
   }
 
  /*
   * Try connecting to the remote server...
   */
 
+  delay       = 5;
+  recoverable = 0;
+  start_time  = time(NULL);
+
   fputs("STATE: +connecting-to-device\n", stderr);
 
   do
   {
-    fprintf(stderr, "INFO: Connecting to %s on port %d...\n", hostname, port);
+    _cupsLangPrintf(stderr, _("INFO: Connecting to %s on port %d...\n"),
+		    hostname, port);
 
     if ((http = httpConnectEncrypt(hostname, port, cupsEncryption())) == NULL)
     {
+      if (job_cancelled)
+	break;
+
       if (getenv("CLASS") != NULL)
       {
        /*
@@ -531,10 +514,9 @@ main(int  argc,				/* I - Number of command-line args */
 	* available printer in the class.
 	*/
 
-        fprintf(stderr,
-	        "INFO: Unable to connect to %s, queuing on next printer in "
-		"class...\n",
-	        hostname);
+        _cupsLangPuts(stderr,
+	              _("INFO: Unable to contact printer, queuing on next "
+			"printer in class...\n"));
 
         if (argc == 6 || strcmp(filename, argv[6]))
 	  unlink(filename);
@@ -551,41 +533,69 @@ main(int  argc,				/* I - Number of command-line args */
       if (errno == ECONNREFUSED || errno == EHOSTDOWN ||
           errno == EHOSTUNREACH)
       {
-	fprintf(stderr,
-	        "INFO: Network host \'%s\' is busy; will retry in 30 "
-		"seconds...\n",
-                hostname);
-	sleep(30);
+        if (contimeout && (time(NULL) - start_time) > contimeout)
+	{
+	  _cupsLangPuts(stderr, _("ERROR: Printer not responding!\n"));
+	  return (CUPS_BACKEND_FAILED);
+	}
+
+        recoverable = 1;
+
+	_cupsLangPrintf(stderr,
+			_("WARNING: recoverable: Network host \'%s\' is busy; "
+			  "will retry in %d seconds...\n"),
+			hostname, delay);
+
+	sleep(delay);
+
+	if (delay < 30)
+	  delay += 5;
       }
       else if (h_errno)
       {
-        fprintf(stderr, "INFO: Unable to lookup host \'%s\' - %s\n",
-	        hostname, hstrerror(h_errno));
-	sleep(30);
+	_cupsLangPrintf(stderr, _("ERROR: Unable to locate printer \'%s\'!\n"),
+			hostname);
+	return (CUPS_BACKEND_STOP);
       }
       else
       {
-	perror("ERROR: Unable to connect to IPP host");
+        recoverable = 1;
+
+        fprintf(stderr, "DEBUG: Connection error: %s\n", strerror(errno));
+	_cupsLangPuts(stderr,
+	              _("ERROR: recoverable: Unable to connect to printer; will "
+			"retry in 30 seconds...\n"));
 	sleep(30);
       }
+
+      if (job_cancelled)
+	break;
     }
   }
   while (http == NULL);
 
+  if (job_cancelled)
+  {
+    if (argc == 6 || strcmp(filename, argv[6]))
+      unlink(filename);
+
+    return (CUPS_BACKEND_FAILED);
+  }
+
   fputs("STATE: -connecting-to-device\n", stderr);
-  fprintf(stderr, "INFO: Connected to %s...\n", hostname);
+  _cupsLangPrintf(stderr, _("INFO: Connected to %s...\n"), hostname);
 
 #ifdef AF_INET6
   if (http->hostaddr->addr.sa_family == AF_INET6)
-    fprintf(stderr, "DEBUG: Connected to [%s]:%d (IPv6)...\n", 
-		    httpAddrString(http->hostaddr, addrname, sizeof(addrname)),
-		    ntohs(http->hostaddr->ipv6.sin6_port));
+    fprintf(stderr, "DEBUG: Connected to [%s]:%d (IPv6)...\n",
+	    httpAddrString(http->hostaddr, addrname, sizeof(addrname)),
+	    ntohs(http->hostaddr->ipv6.sin6_port));
   else
 #endif /* AF_INET6 */
     if (http->hostaddr->addr.sa_family == AF_INET)
       fprintf(stderr, "DEBUG: Connected to %s:%d (IPv4)...\n",
-		      httpAddrString(http->hostaddr, addrname, sizeof(addrname)),
-		      ntohs(http->hostaddr->ipv4.sin_port));
+	      httpAddrString(http->hostaddr, addrname, sizeof(addrname)),
+	      ntohs(http->hostaddr->ipv4.sin_port));
 
  /*
   * Build a URI for the printer and fill the standard IPP attributes for
@@ -637,9 +647,25 @@ main(int  argc,				/* I - Number of command-line args */
       if (ipp_status == IPP_PRINTER_BUSY ||
 	  ipp_status == IPP_SERVICE_UNAVAILABLE)
       {
-	fputs("INFO: Printer busy; will retry in 10 seconds...\n", stderr);
-        report_printer_state(supported);
-	sleep(10);
+        if (contimeout && (time(NULL) - start_time) > contimeout)
+	{
+	  _cupsLangPuts(stderr, _("ERROR: Printer not responding!\n"));
+	  return (CUPS_BACKEND_FAILED);
+	}
+
+        recoverable = 1;
+
+	_cupsLangPrintf(stderr,
+			_("WARNING: recoverable: Network host \'%s\' is busy; "
+			  "will retry in %d seconds...\n"),
+			hostname, delay);
+
+        report_printer_state(supported, 0);
+
+	sleep(delay);
+
+	if (delay < 30)
+	  delay += 5;
       }
       else if ((ipp_status == IPP_BAD_REQUEST ||
 	        ipp_status == IPP_VERSION_NOT_SUPPORTED) && version == 1)
@@ -648,14 +674,15 @@ main(int  argc,				/* I - Number of command-line args */
 	* Switch to IPP/1.0...
 	*/
 
-	fputs("INFO: Printer does not support IPP/1.1, trying IPP/1.0...\n",
-	      stderr);
+	_cupsLangPuts(stderr,
+	              _("INFO: Printer does not support IPP/1.1, trying "
+		        "IPP/1.0...\n"));
 	version = 0;
 	httpReconnect(http);
       }
       else if (ipp_status == IPP_NOT_FOUND)
       {
-        fputs("ERROR: Destination printer does not exist!\n", stderr);
+        _cupsLangPuts(stderr, _("ERROR: Destination printer does not exist!\n"));
 
 	if (supported)
           ippDelete(supported);
@@ -664,8 +691,9 @@ main(int  argc,				/* I - Number of command-line args */
       }
       else
       {
-	fprintf(stderr, "ERROR: Unable to get printer status (%s)!\n",
-	        ippErrorString(ipp_status));
+	_cupsLangPrintf(stderr,
+	                _("ERROR: Unable to get printer status (%s)!\n"),
+			cupsLastErrorString());
         sleep(10);
       }
 
@@ -698,7 +726,7 @@ main(int  argc,				/* I - Number of command-line args */
 	        format_sup->values[i].string.text);
     }
 
-    report_printer_state(supported);
+    report_printer_state(supported, 0);
   }
   while (ipp_status > IPP_OK_CONFLICT);
 
@@ -727,10 +755,9 @@ main(int  argc,				/* I - Number of command-line args */
       * available printer in the class.
       */
 
-      fprintf(stderr,
-              "INFO: Unable to queue job on %s, queuing on next printer in "
-	      "class...\n",
-	      hostname);
+      _cupsLangPuts(stderr,
+                    _("INFO: Unable to contact printer, queuing on next "
+		      "printer in class...\n"));
 
       ippDelete(supported);
       httpClose(http);
@@ -748,6 +775,18 @@ main(int  argc,				/* I - Number of command-line args */
     }
   }
 
+  if (recoverable)
+  {
+   /*
+    * If we've shown a recoverable error make sure the printer proxies
+    * have a chance to see the recovered message. Not pretty but
+    * necessary for now...
+    */
+
+    fputs("INFO: recovered: \n", stderr);
+    sleep(5);
+  }
+
  /*
   * See if the printer supports multiple copies...
   */
@@ -823,8 +862,7 @@ main(int  argc,				/* I - Number of command-line args */
     num_options = cupsParseOptions(argv[5], 0, &options);
 
 #ifdef __APPLE__
-    if (content_type != NULL &&
-        !strcasecmp(content_type, "application/pictwps") && num_files == 1)
+    if (!strcasecmp(content_type, "application/pictwps") && num_files == 1)
     {
       if (format_sup != NULL)
       {
@@ -850,25 +888,26 @@ main(int  argc,				/* I - Number of command-line args */
 	* number of copies to 1...
 	*/
 
-	content_type     = "application/postscript";
-	copies           = 1;
-	copies_remaining = 1;
+	final_content_type = "application/postscript";
+	copies             = 1;
+	copies_remaining   = 1;
+        send_options       = 0;
       }
     }
 #endif /* __APPLE__ */
 
-    if (content_type != NULL && format_sup != NULL)
+    if (format_sup != NULL)
     {
       for (i = 0; i < format_sup->num_values; i ++)
-        if (!strcasecmp(content_type, format_sup->values[i].string.text))
+        if (!strcasecmp(final_content_type, format_sup->values[i].string.text))
           break;
 
       if (i < format_sup->num_values)
-        num_options = cupsAddOption("document-format", content_type,
-	                            num_options, &options);
+        ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_MIMETYPE,
+	             "document-format", NULL, final_content_type);
     }
 
-    if (copies_sup && version > 0)
+    if (copies_sup && version > 0 && send_options)
     {
      /*
       * Only send options if the destination printer supports the copies
@@ -918,7 +957,8 @@ main(int  argc,				/* I - Number of command-line args */
       if (ipp_status == IPP_SERVICE_UNAVAILABLE ||
 	  ipp_status == IPP_PRINTER_BUSY)
       {
-	fputs("INFO: Printer is busy; retrying print job...\n", stderr);
+        _cupsLangPuts(stderr,
+	              _("INFO: Printer busy; will retry in 10 seconds...\n"));
 	sleep(10);
       }
       else if ((ipp_status == IPP_BAD_REQUEST ||
@@ -928,25 +968,28 @@ main(int  argc,				/* I - Number of command-line args */
 	* Switch to IPP/1.0...
 	*/
 
-	fputs("INFO: Printer does not support IPP/1.1, trying IPP/1.0...\n",
-	      stderr);
+	_cupsLangPuts(stderr,
+	              _("INFO: Printer does not support IPP/1.1, trying "
+		        "IPP/1.0...\n"));
 	version = 0;
 	httpReconnect(http);
       }
       else
-        fprintf(stderr, "ERROR: Print file was not accepted (%s)!\n",
-	        cupsLastErrorString());
+        _cupsLangPrintf(stderr, _("ERROR: Print file was not accepted (%s)!\n"),
+			cupsLastErrorString());
     }
     else if ((job_id_attr = ippFindAttribute(response, "job-id",
                                              IPP_TAG_INTEGER)) == NULL)
     {
-      fputs("NOTICE: Print file accepted - job ID unknown.\n", stderr);
+      _cupsLangPuts(stderr,
+                    _("NOTICE: Print file accepted - job ID unknown.\n"));
       job_id = 0;
     }
     else
     {
       job_id = job_id_attr->values[0].integer;
-      fprintf(stderr, "NOTICE: Print file accepted - job ID %d.\n", job_id);
+      _cupsLangPrintf(stderr, _("NOTICE: Print file accepted - job ID %d.\n"),
+                      job_id);
     }
 
     ippDelete(response);
@@ -984,8 +1027,9 @@ main(int  argc,				/* I - Number of command-line args */
 	{
 	  ipp_status = cupsLastError();
 
-	  fprintf(stderr, "ERROR: Unable to add file %d to job: %s\n",
-	          job_id, cupsLastErrorString());
+	  _cupsLangPrintf(stderr,
+			  _("ERROR: Unable to add file %d to job: %s\n"),
+			  job_id, cupsLastErrorString());
 	  break;
 	}
       }
@@ -1009,7 +1053,7 @@ main(int  argc,				/* I - Number of command-line args */
     if (!job_id || !waitjob)
       continue;
 
-    fputs("INFO: Waiting for job to complete...\n", stderr);
+    _cupsLangPuts(stderr, _("INFO: Waiting for job to complete...\n"));
 
     for (; !job_cancelled;)
     {
@@ -1063,8 +1107,9 @@ main(int  argc,				/* I - Number of command-line args */
 	{
 	  ippDelete(response);
 
-          fprintf(stderr, "ERROR: Unable to get job %d attributes (%s)!\n",
-	          job_id, ippErrorString(ipp_status));
+          _cupsLangPrintf(stderr,
+			  _("ERROR: Unable to get job %d attributes (%s)!\n"),
+			  job_id, cupsLastErrorString());
           break;
 	}
       }
@@ -1078,8 +1123,7 @@ main(int  argc,				/* I - Number of command-line args */
           * Stop polling if the job is finished or pending-held...
 	  */
 
-          if (job_state->values[0].integer > IPP_JOB_PROCESSING ||
-	      job_state->values[0].integer == IPP_JOB_HELD)
+          if (job_state->values[0].integer > IPP_JOB_STOPPED)
 	  {
 	    if ((job_sheets = ippFindAttribute(response, 
 	                                       "job-media-sheets-completed",
@@ -1099,7 +1143,7 @@ main(int  argc,				/* I - Number of command-line args */
       * Check the printer state and report it if necessary...
       */
 
-      check_printer_state(http, uri, resource, argv[2], version);
+      check_printer_state(http, uri, resource, argv[2], version, job_id);
 
      /*
       * Wait 10 seconds before polling again...
@@ -1120,7 +1164,7 @@ main(int  argc,				/* I - Number of command-line args */
   * Check the printer state and report it if necessary...
   */
 
-  check_printer_state(http, uri, resource, argv[2], version);
+  check_printer_state(http, uri, resource, argv[2], version, job_id);
 
  /*
   * Free memory...
@@ -1154,7 +1198,20 @@ main(int  argc,				/* I - Number of command-line args */
   * Return the queue status...
   */
 
-  return (ipp_status > IPP_OK_CONFLICT ? CUPS_BACKEND_FAILED : CUPS_BACKEND_OK);
+  if (ipp_status == IPP_NOT_AUTHORIZED)
+  {
+   /*
+    * Authorization failures here mean that we need Kerberos.  Username +
+    * password authentication is handled in the password_cb function.
+    */
+
+    fputs("ATTR: auth-info-required=negotiate\n", stderr);
+    return (CUPS_BACKEND_AUTH_REQUIRED);
+  }
+  else if (ipp_status > IPP_OK_CONFLICT)
+    return (CUPS_BACKEND_FAILED);
+  else
+    return (CUPS_BACKEND_OK);
 }
 
 
@@ -1173,7 +1230,7 @@ cancel_job(http_t     *http,		/* I - HTTP connection */
   ipp_t	*request;			/* Cancel-Job request */
 
 
-  fputs("INFO: Cancelling print job...\n", stderr);
+  _cupsLangPuts(stderr, _("INFO: Canceling print job...\n"));
 
   request = ippNewRequest(IPP_CANCEL_JOB);
   request->request.op.version[1] = version;
@@ -1193,8 +1250,8 @@ cancel_job(http_t     *http,		/* I - HTTP connection */
   ippDelete(cupsDoRequest(http, request, resource));
 
   if (cupsLastError() > IPP_OK_CONFLICT)
-    fprintf(stderr, "ERROR: Unable to cancel job %d: %s\n", id,
-            cupsLastErrorString());
+    _cupsLangPrintf(stderr, _("ERROR: Unable to cancel job %d: %s\n"), id,
+		    cupsLastErrorString());
 }
 
 
@@ -1208,10 +1265,16 @@ check_printer_state(
     const char  *uri,			/* I - Printer URI */
     const char  *resource,		/* I - Resource path */
     const char  *user,			/* I - Username, if any */
-    int         version)		/* I - IPP version */
+    int         version,		/* I - IPP version */
+    int         job_id)			/* I - Current job ID */
 {
   ipp_t	*request,			/* IPP request */
 	*response;			/* IPP response */
+  static const char * const attrs[] =	/* Attributes we want */
+  {
+    "printer-state-message",
+    "printer-state-reasons"
+  };
 
 
  /*
@@ -1228,8 +1291,9 @@ check_printer_state(
     ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_NAME,
                  "requesting-user-name", NULL, user);
 
-  ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_KEYWORD,
-               "requested-attributes", NULL, "printer-state-reasons");
+  ippAddStrings(request, IPP_TAG_OPERATION, IPP_TAG_KEYWORD,
+                "requested-attributes",
+		(int)(sizeof(attrs) / sizeof(attrs[0])), NULL, attrs);
 
  /*
   * Do the request...
@@ -1237,7 +1301,7 @@ check_printer_state(
 
   if ((response = cupsDoRequest(http, request, resource)) != NULL)
   {
-    report_printer_state(response);
+    report_printer_state(response, job_id);
     ippDelete(response);
   }
 }
@@ -1268,20 +1332,25 @@ compress_files(int  num_files,		/* I - Number of files */
   {
     if ((fd = cupsTempFd(filename, sizeof(filename))) < 0)
     {
-      perror("ERROR: Unable to create temporary compressed print file");
+      _cupsLangPrintf(stderr,
+		      _("ERROR: Unable to create temporary compressed print "
+		        "file: %s\n"), strerror(errno));
       exit(CUPS_BACKEND_FAILED);
     }
 
     if ((out = cupsFileOpenFd(fd, "w9")) == NULL)
     {
-      perror("ERROR: Unable to open temporary compressed print file");
+      _cupsLangPrintf(stderr,
+		      _("ERROR: Unable to open temporary compressed print "
+		        "file: %s\n"), strerror(errno));
       exit(CUPS_BACKEND_FAILED);
     }
 
     if ((in = cupsFileOpen(files[i], "r")) == NULL)
     {
-      fprintf(stderr, "ERROR: Unable to open print file \"%s\": %s\n",
-              files[i], strerror(errno));
+      _cupsLangPrintf(stderr,
+                      _("ERROR: Unable to open print file \"%s\": %s\n"),
+		      files[i], strerror(errno));
       cupsFileClose(out);
       exit(CUPS_BACKEND_FAILED);
     }
@@ -1290,8 +1359,9 @@ compress_files(int  num_files,		/* I - Number of files */
     while ((bytes = cupsFileRead(in, buffer, sizeof(buffer))) > 0)
       if (cupsFileWrite(out, buffer, bytes) < bytes)
       {
-        fprintf(stderr, "ERROR: Unable to write " CUPS_LLFMT " bytes to \"%s\": %s\n",
-	        CUPS_LLCAST bytes, filename, strerror(errno));
+        _cupsLangPrintf(stderr,
+		        _("ERROR: Unable to write %d bytes to \"%s\": %s\n"),
+			(int)bytes, filename, strerror(errno));
         cupsFileClose(in);
         cupsFileClose(out);
 	exit(CUPS_BACKEND_FAILED);
@@ -1324,8 +1394,12 @@ password_cb(const char *prompt)		/* I - Prompt (not used) */
 {
   (void)prompt;
 
-  if (password)
+  if (password && password_tries < 3)
+  {
+    password_tries ++;
+
     return (password);
+  }
   else
   {
    /*
@@ -1341,6 +1415,8 @@ password_cb(const char *prompt)		/* I - Prompt (not used) */
       unlink(pstmpname);
 #endif /* __APPLE__ */
 
+    fputs("ATTR: auth-info-required=username,password\n", stderr);
+
     exit(CUPS_BACKEND_AUTH_REQUIRED);
 
     return (NULL);			/* Eliminate compiler warning */
@@ -1353,102 +1429,114 @@ password_cb(const char *prompt)		/* I - Prompt (not used) */
  */
 
 static int				/* O - Number of reasons shown */
-report_printer_state(ipp_t *ipp)	/* I - IPP response */
+report_printer_state(ipp_t *ipp,	/* I - IPP response */
+                     int   job_id)	/* I - Current job ID */
 {
   int			i;		/* Looping var */
   int			count;		/* Count of reasons shown... */
-  ipp_attribute_t	*reasons;	/* printer-state-reasons */
+  ipp_attribute_t	*psm,		/* pritner-state-message */
+			*reasons;	/* printer-state-reasons */
   const char		*reason;	/* Current reason */
   const char		*message;	/* Message to show */
   char			unknown[1024];	/* Unknown message string */
   const char		*prefix;	/* Prefix for STATE: line */
   char			state[1024];	/* State string */
+  cups_lang_t		*language;	/* Current localization */
 
 
+  if ((psm = ippFindAttribute(ipp, "printer-state-message",
+                              IPP_TAG_TEXT)) != NULL)
+    fprintf(stderr, "INFO: %s\n", psm->values[0].string.text);
+
   if ((reasons = ippFindAttribute(ipp, "printer-state-reasons",
                                   IPP_TAG_KEYWORD)) == NULL)
     return (0);
 
   state[0] = '\0';
   prefix   = "STATE: ";
+  language = cupsLangDefault();
 
   for (i = 0, count = 0; i < reasons->num_values; i ++)
   {
     reason = reasons->values[i].string.text;
 
-    strlcat(state, prefix, sizeof(state));
-    strlcat(state, reason, sizeof(state));
+    if (job_id == 0 || strcmp(reason, "paused"))
+    {
+      strlcat(state, prefix, sizeof(state));
+      strlcat(state, reason, sizeof(state));
+
+      prefix  = ",";
+    }
 
-    prefix  = ",";
     message = "";
 
     if (!strncmp(reason, "media-needed", 12))
-      message = "Media tray needs to be filled.";
+      message = _("Media tray needs to be filled.");
     else if (!strncmp(reason, "media-jam", 9))
-      message = "Media jam!";
+      message = _("Media jam!");
     else if (!strncmp(reason, "moving-to-paused", 16) ||
              !strncmp(reason, "paused", 6) ||
 	     !strncmp(reason, "shutdown", 8))
-      message = "Printer off-line.";
+      message = _("Printer off-line.");
     else if (!strncmp(reason, "toner-low", 9))
-      message = "Toner low.";
+      message = _("Toner low.");
     else if (!strncmp(reason, "toner-empty", 11))
-      message = "Out of toner!";
+      message = _("Out of toner!");
     else if (!strncmp(reason, "cover-open", 10))
-      message = "Cover open.";
+      message = _("Cover open.");
     else if (!strncmp(reason, "interlock-open", 14))
-      message = "Interlock open.";
+      message = _("Interlock open.");
     else if (!strncmp(reason, "door-open", 9))
-      message = "Door open.";
+      message = _("Door open.");
     else if (!strncmp(reason, "input-tray-missing", 18))
-      message = "Media tray missing!";
+      message = _("Media tray missing!");
     else if (!strncmp(reason, "media-low", 9))
-      message = "Media tray almost empty.";
+      message = _("Media tray almost empty.");
     else if (!strncmp(reason, "media-empty", 11))
-      message = "Media tray empty!";
+      message = _("Media tray empty!");
     else if (!strncmp(reason, "output-tray-missing", 19))
-      message = "Output tray missing!";
+      message = _("Output tray missing!");
     else if (!strncmp(reason, "output-area-almost-full", 23))
-      message = "Output bin almost full.";
+      message = _("Output bin almost full.");
     else if (!strncmp(reason, "output-area-full", 16))
-      message = "Output bin full!";
+      message = _("Output bin full!");
     else if (!strncmp(reason, "marker-supply-low", 17))
-      message = "Ink/toner almost empty.";
+      message = _("Ink/toner almost empty.");
     else if (!strncmp(reason, "marker-supply-empty", 19))
-      message = "Ink/toner empty!";
+      message = _("Ink/toner empty!");
     else if (!strncmp(reason, "marker-waste-almost-full", 24))
-      message = "Ink/toner waste bin almost full.";
+      message = _("Ink/toner waste bin almost full.");
     else if (!strncmp(reason, "marker-waste-full", 17))
-      message = "Ink/toner waste bin full!";
+      message = _("Ink/toner waste bin full!");
     else if (!strncmp(reason, "fuser-over-temp", 15))
-      message = "Fuser temperature high!";
+      message = _("Fuser temperature high!");
     else if (!strncmp(reason, "fuser-under-temp", 16))
-      message = "Fuser temperature low!";
+      message = _("Fuser temperature low!");
     else if (!strncmp(reason, "opc-near-eol", 12))
-      message = "OPC almost at end-of-life.";
+      message = _("OPC almost at end-of-life.");
     else if (!strncmp(reason, "opc-life-over", 13))
-      message = "OPC at end-of-life!";
+      message = _("OPC at end-of-life!");
     else if (!strncmp(reason, "developer-low", 13))
-      message = "Developer almost empty.";
+      message = _("Developer almost empty.");
     else if (!strncmp(reason, "developer-empty", 15))
-      message = "Developer empty!";
+      message = _("Developer empty!");
     else if (strstr(reason, "error") != NULL)
     {
       message = unknown;
 
-      snprintf(unknown, sizeof(unknown), "Unknown printer error (%s)!",
+      snprintf(unknown, sizeof(unknown), _("Unknown printer error (%s)!"),
                reason);
     }
 
-    if (message)
+    if (message[0])
     {
       count ++;
       if (strstr(reasons->values[i].string.text, "error"))
-        fprintf(stderr, "ERROR: %s\n", message);
+        fprintf(stderr, "ERROR: %s\n", _cupsLangString(language, message));
       else if (strstr(reasons->values[i].string.text, "warning"))
-        fprintf(stderr, "WARNING: %s\n", message);
+        fprintf(stderr, "WARNING: %s\n", _cupsLangString(language, message));
       else
-        fprintf(stderr, "INFO: %s\n", message);
+        fprintf(stderr, "INFO: %s\n", _cupsLangString(language, message));
     }
   }
 
@@ -1490,15 +1578,16 @@ run_pictwps_filter(char       **argv,	/* I - Command-line arguments */
   printer = getenv("PRINTER");
   if (!printer)
   {
-    fputs("ERROR: PRINTER environment variable not defined!\n", stderr);
+    _cupsLangPuts(stderr,
+                  _("ERROR: PRINTER environment variable not defined!\n"));
     return (-1);
   }
 
   if ((ppdfile = cupsGetPPD(printer)) == NULL)
   {
-    fprintf(stderr, "ERROR: Unable to get PPD file for printer \"%s\" - %s.\n",
-            printer, ippErrorString(cupsLastError()));
-    /*return (-1);*/
+    _cupsLangPrintf(stderr,
+		    _("ERROR: Unable to get PPD file for printer \"%s\" - "
+		      "%s.\n"), printer, cupsLastErrorString());
   }
   else
   {
@@ -1512,8 +1601,8 @@ run_pictwps_filter(char       **argv,	/* I - Command-line arguments */
 
   if ((fd = cupsTempFd(pstmpname, sizeof(pstmpname))) < 0)
   {
-    fprintf(stderr, "ERROR: Unable to create temporary file - %s.\n",
-            strerror(errno));
+    _cupsLangPrintf(stderr, _("ERROR: Unable to create temporary file - %s.\n"),
+		    strerror(errno));
     if (ppdfile)
       unlink(ppdfile);
     return (-1);
@@ -1569,7 +1658,8 @@ run_pictwps_filter(char       **argv,	/* I - Command-line arguments */
 
     execlp("pictwpstops", printer, argv[1], argv[2], argv[3], argv[4], argv[5],
            filename, NULL);
-    perror("ERROR: Unable to exec pictwpstops");
+    _cupsLangPrintf(stderr, _("ERROR: Unable to exec pictwpstops: %s\n"),
+		    strerror(errno));
     return (errno);
   }
 
@@ -1581,7 +1671,8 @@ run_pictwps_filter(char       **argv,	/* I - Command-line arguments */
     * Error!
     */
 
-    perror("ERROR: Unable to fork pictwpstops");
+    _cupsLangPrintf(stderr, _("ERROR: Unable to fork pictwpstops: %s\n"),
+		    strerror(errno));
     unlink(filename);
     if (ppdfile)
       unlink(ppdfile);
@@ -1594,7 +1685,8 @@ run_pictwps_filter(char       **argv,	/* I - Command-line arguments */
 
   if (wait(&status) < 0)
   {
-    perror("ERROR: Unable to wait for pictwpstops");
+    _cupsLangPrintf(stderr, _("ERROR: Unable to wait for pictwpstops: %s\n"),
+		    strerror(errno));
     close(fd);
     unlink(filename);
     if (ppdfile)
@@ -1610,11 +1702,11 @@ run_pictwps_filter(char       **argv,	/* I - Command-line arguments */
   if (status)
   {
     if (status >= 256)
-      fprintf(stderr, "ERROR: pictwpstops exited with status %d!\n",
-              status / 256);
+      _cupsLangPrintf(stderr, _("ERROR: pictwpstops exited with status %d!\n"),
+		      status / 256);
     else
-      fprintf(stderr, "ERROR: pictwpstops exited on signal %d!\n",
-              status);
+      _cupsLangPrintf(stderr, _("ERROR: pictwpstops exited on signal %d!\n"),
+		      status);
 
     unlink(filename);
     return (status);
@@ -1666,5 +1758,5 @@ sigterm_handler(int sig)		/* I - Signal */
 
 
 /*
- * End of "$Id: ipp.c 6214 2007-01-23 17:01:48Z mike $".
+ * End of "$Id: ipp.c 6967 2007-09-17 23:30:52Z mike $".
  */
