@@ -1,25 +1,16 @@
 /*
- * "$Id: lpr.c 5235 2006-03-06 13:02:23Z mike $"
+ * "$Id: lpr.c 6649 2007-07-11 21:46:42Z mike $"
  *
  *   "lpr" command for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2006 by Easy Software Products.
+ *   Copyright 2007 by Apple Inc.
+ *   Copyright 1997-2007 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "LICENSE.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ *   which should have been included with this file.  If this file is
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
  * Contents:
  *
@@ -87,13 +78,14 @@ main(int  argc,				/* I - Number of command-line arguments */
   ssize_t	bytes;			/* Bytes copied */
   off_t		filesize;		/* Size of temp file */
   int		temp;			/* Temporary file descriptor */
-  cups_lang_t	*language;		/* Language information */
 #if defined(HAVE_SIGACTION) && !defined(HAVE_SIGSET)
   struct sigaction action;		/* Signal action */
   struct sigaction oldaction;		/* Old signal action */
 #endif /* HAVE_SIGACTION && !HAVE_SIGSET */
 
 
+  _cupsSetLocale(argv);
+
   deletefile  = 0;
   printer     = NULL;
   num_dests   = 0;
@@ -102,7 +94,6 @@ main(int  argc,				/* I - Number of command-line arguments */
   options     = NULL;
   num_files   = 0;
   title       = NULL;
-  language    = cupsLangDefault();
 
   for (i = 1; i < argc; i ++)
     if (argv[i][0] == '-')
@@ -232,7 +223,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 
 	      snprintf(email, sizeof(email), "mailto:%s@%s", cupsUser(),
 	               httpGetHostname(NULL, buffer, sizeof(buffer)));
-	      num_options = cupsAddOption("notify-recipient", email,
+	      num_options = cupsAddOption("notify-recipient-uri", email,
 	                                  num_options, &options);
 	    }
 	    break;
@@ -529,5 +520,5 @@ sighandler(int s)			/* I - Signal number */
 
 
 /*
- * End of "$Id: lpr.c 5235 2006-03-06 13:02:23Z mike $".
+ * End of "$Id: lpr.c 6649 2007-07-11 21:46:42Z mike $".
  */
