@@ -1,25 +1,16 @@
 /*
- * "$Id: emit.c 5700 2006-06-26 19:20:39Z mike $"
+ * "$Id: emit.c 6649 2007-07-11 21:46:42Z mike $"
  *
  *   PPD code emission routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2006 by Easy Software Products, all rights reserved.
+ *   Copyright 2007 by Apple Inc.
+ *   Copyright 1997-2007 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "LICENSE.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ *   which should have been included with this file.  If this file is
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
  *   PostScript is a trademark of Adobe Systems, Inc.
  *
@@ -291,7 +282,11 @@ ppdEmitFd(ppd_file_t    *ppd,		/* I - PPD file record */
 
     while (buflength > 0)
     {
+#ifdef WIN32
+      if ((bytes = (ssize_t)write(fd, bufptr, (unsigned)buflength)) < 0)
+#else
       if ((bytes = write(fd, bufptr, buflength)) < 0)
+#endif /* WIN32 */
       {
         if (errno == EAGAIN || errno == EINTR)
 	  continue;
@@ -351,6 +346,15 @@ ppdEmitJCL(ppd_file_t *ppd,		/* I - PPD file record */
     * of the PJL commands to initialize PJL processing.
     */
 
+    ppd_attr_t	*charset;		/* PJL charset */
+
+
+    if ((charset = ppdFindAttr(ppd, "cupsPJLCharset", NULL)) != NULL)
+    {
+      if (!charset->value || strcasecmp(charset->value, "UTF-8"))
+        charset = NULL;
+    }
+
     fputs("\033%-12345X@PJL\n", fp);
     for (ptr = ppd->jcl_begin + 9; *ptr;)
       if (!strncmp(ptr, "@PJL JOB", 8))
@@ -391,8 +395,8 @@ ppdEmitJCL(ppd_file_t *ppd,		/* I - PPD file record */
       title = ptr + 1;
 
    /*
-    * Replace double quotes with single quotes so that the title
-    * does not cause a PJL syntax error.
+    * Replace double quotes with single quotes and 8-bit characters with
+    * question marks so that the title does not cause a PJL syntax error.
     */
 
     strlcpy(temp, title, sizeof(temp));
@@ -400,6 +404,8 @@ ppdEmitJCL(ppd_file_t *ppd,		/* I - PPD file record */
     for (ptr = temp; *ptr; ptr ++)
       if (*ptr == '\"')
         *ptr = '\'';
+      else if (charset && (*ptr & 128))
+        *ptr = '?';
 
    /*
     * Send PJL JOB and PJL RDYMSG commands before we enter PostScript mode...
@@ -723,7 +729,7 @@ ppdEmitString(ppd_file_t    *ppd,	/* I - PPD file record */
 	else
 	  pos = 4;
 
-	values[pos] = orientation;
+	values[pos] = (float)orientation;
 
         for (pos = 0; pos < 5; pos ++)
 	{
@@ -809,7 +815,7 @@ ppdEmitString(ppd_file_t    *ppd,	/* I - PPD file record */
 
       if (choices[i]->code && choices[i]->code[0])
       {
-        j = strlen(choices[i]->code);
+        j = (int)strlen(choices[i]->code);
 	memcpy(bufptr, choices[i]->code, j);
 	bufptr += j;
 
@@ -940,5 +946,5 @@ ppd_sort(ppd_choice_t **c1,	/* I - First choice */
 
 
 /*
- * End of "$Id: emit.c 5700 2006-06-26 19:20:39Z mike $".
+ * End of "$Id: emit.c 6649 2007-07-11 21:46:42Z mike $".
  */
