@@ -1,25 +1,16 @@
 /*
- * "$Id: ppd.c 6188 2007-01-10 16:23:06Z mike $"
+ * "$Id: ppd.c 6942 2007-09-10 22:55:00Z mike $"
  *
  *   PPD file routines for the Common UNIX Printing System (CUPS).
  *
+ *   Copyright 2007 by Apple Inc.
  *   Copyright 1997-2007 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "LICENSE.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ *   which should have been included with this file.  If this file is
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
  *   PostScript is a trademark of Adobe Systems, Inc.
  *
@@ -48,6 +39,8 @@
  *   ppd_add_choice()       - Add a choice to an option.
  *   ppd_add_size()         - Add a page size.
  *   ppd_compare_attrs()    - Compare two attributes.
+ *   ppd_compare_choices()  - Compare two choices...
+ *   ppd_compare_consts()   - Compare two constraints.
  *   ppd_compare_coptions() - Compare two custom options.
  *   ppd_compare_cparams()  - Compare two custom parameters.
  *   ppd_compare_options()  - Compare two options.
@@ -58,6 +51,7 @@
  *   ppd_get_cparam()       - Get a custom parameter record.
  *   ppd_get_group()        - Find or create the named group as needed.
  *   ppd_get_option()       - Find or create the named option as needed.
+ *   ppd_hash_option()      - Generate a hash of the option name...
  *   ppd_read()             - Read a line from a PPD file, skipping comment
  *                            lines as necessary.
  */
@@ -90,6 +84,8 @@
 #define PPD_TEXT	4		/* Line contained human-readable text */
 #define PPD_STRING	8		/* Line contained a string or code */
 
+#define PPD_HASHSIZE	512		/* Size of hash */
+
 
 /*
  * Local functions...
@@ -101,6 +97,8 @@ static ppd_attr_t	*ppd_add_attr(ppd_file_t *ppd, const char *name,
 static ppd_choice_t	*ppd_add_choice(ppd_option_t *option, const char *name);
 static ppd_size_t	*ppd_add_size(ppd_file_t *ppd, const char *name);
 static int		ppd_compare_attrs(ppd_attr_t *a, ppd_attr_t *b);
+static int		ppd_compare_choices(ppd_choice_t *a, ppd_choice_t *b);
+static int		ppd_compare_consts(ppd_const_t *a, ppd_const_t *b);
 static int		ppd_compare_coptions(ppd_coption_t *a,
 			                     ppd_coption_t *b);
 static int		ppd_compare_cparams(ppd_cparam_t *a, ppd_cparam_t *b);
@@ -116,6 +114,7 @@ static ppd_group_t	*ppd_get_group(ppd_file_t *ppd, const char *name,
 			               const char *text, _cups_globals_t *cg,
 				       cups_encoding_t encoding);
 static ppd_option_t	*ppd_get_option(ppd_group_t *group, const char *name);
+static int		ppd_hash_option(ppd_option_t *option);
 static int		ppd_read(cups_file_t *fp, char *keyword, char *option,
 			         char *text, char **string, int ignoreblank,
 				 _cups_globals_t *cg);
@@ -184,6 +183,7 @@ ppdClose(ppd_file_t *ppd)		/* I - PPD file record */
   }
 
   cupsArrayDelete(ppd->options);
+  cupsArrayDelete(ppd->marked);
 
  /*
   * Free any page sizes...
@@ -573,7 +573,7 @@ ppdOpen2(cups_file_t *fp)		/* I - File to read from */
     return (NULL);
   }
 
-  ppd->language_level = 1;
+  ppd->language_level = 2;
   ppd->color_device   = 0;
   ppd->colorspace     = PPD_CS_GRAY;
   ppd->landscape      = -90;
@@ -1003,7 +1003,7 @@ ppdOpen2(cups_file_t *fp)		/* I - File to read from */
       for (i = 0, sptr = string; i < 4; i ++)
         ppd->custom_margins[i] = (float)_cupsStrScand(sptr, &sptr, loc);
     }
-    else if (!strncmp(keyword, "Custom", 6) && !strcmp(name, "True"))
+    else if (!strncmp(keyword, "Custom", 6) && !strcmp(name, "True") && !option)
     {
       DEBUG_puts("Processing Custom option...");
 
@@ -1013,11 +1013,15 @@ ppdOpen2(cups_file_t *fp)		/* I - File to read from */
 
       if ((option = ppdFindOption(ppd, keyword + 6)) == NULL)
       {
+        int		groupidx = -1;	/* Index for current group */
 	ppd_group_t	*gtemp;		/* Temporary group */
 
 
         DEBUG_printf(("%s option not found for %s...\n", keyword + 6, keyword));
 
+        if (group)
+          groupidx = group - ppd->groups; /* Save index for current group */
+
 	if ((gtemp = ppd_get_group(ppd, "General", _("General"), cg,
 	                           encoding)) == NULL)
 	{
@@ -1026,6 +1030,9 @@ ppdOpen2(cups_file_t *fp)		/* I - File to read from */
 	  goto error;
 	}
 
+        if (group)
+          group = ppd->groups + groupidx; /* Restore group pointer */
+
 	if ((option = ppd_get_option(gtemp, keyword + 6)) == NULL)
 	{
 	  DEBUG_printf(("Unable to get %s option!\n", keyword + 6));
@@ -1076,6 +1083,41 @@ ppdOpen2(cups_file_t *fp)		/* I - File to read from */
 	*/
 
 	ppd_add_size(ppd, "Custom");
+
+	if ((option = ppdFindOption(ppd, "PageRegion")) == NULL)
+	{
+	  int		groupidx = -1;	/* Index to current group */
+	  ppd_group_t	*gtemp;		/* Temporary group */
+
+          if (group)
+            groupidx = group - ppd->groups; /* Save index for current group */
+
+	  if ((gtemp = ppd_get_group(ppd, "General", _("General"), cg,
+				     encoding)) == NULL)
+	  {
+	    DEBUG_puts("Unable to get general group!");
+
+	    goto error;
+	  }
+
+          if (group)
+            group = ppd->groups + groupidx; /* Restore group pointer */
+
+	  option = ppd_get_option(gtemp, "PageRegion");
+        }
+
+	if ((choice = ppd_add_choice(option, "Custom")) == NULL)
+	{
+	  DEBUG_puts("Unable to add Custom choice!");
+
+	  cg->ppd_status = PPD_ALLOC_ERROR;
+
+	  goto error;
+	}
+
+	strlcpy(choice->text, text[0] ? text : _("Custom"),
+		sizeof(choice->text));
+        option = NULL;
       }
     }
     else if (!strcmp(keyword, "LandscapeOrientation"))
@@ -1171,6 +1213,8 @@ ppdOpen2(cups_file_t *fp)		/* I - File to read from */
       * Add an option record to the current sub-group, group, or file...
       */
 
+      DEBUG_printf(("name=\"%s\" (%d)\n", name, strlen(name)));
+
       if (name[0] == '*')
         _cups_strcpy(name, name + 1); /* Eliminate leading asterisk */
 
@@ -1523,10 +1567,10 @@ ppdOpen2(cups_file_t *fp)		/* I - File to read from */
              !strcmp(keyword, "NonUIConstraints"))
     {
       if (ppd->num_consts == 0)
-	constraint = calloc(1, sizeof(ppd_const_t));
+	constraint = calloc(2, sizeof(ppd_const_t));
       else
 	constraint = realloc(ppd->consts,
-	                     (ppd->num_consts + 1) * sizeof(ppd_const_t));
+	                     (ppd->num_consts + 2) * sizeof(ppd_const_t));
 
       if (constraint == NULL)
       {
@@ -1687,6 +1731,34 @@ ppdOpen2(cups_file_t *fp)		/* I - File to read from */
 	    break;
       }
 
+     /*
+      * For CustomPageSize and InputSlot/ManualFeed, create a duplicate
+      * constraint for PageRegion...
+      */
+
+      if (!strcasecmp(constraint->option1, "CustomPageSize") &&
+          (!strcasecmp(constraint->option2, "InputSlot") ||
+	   !strcasecmp(constraint->option2, "ManualFeed")))
+      {
+        ppd->num_consts ++;
+
+        strcpy(constraint[1].option1, "PageRegion");
+	strcpy(constraint[1].choice1, "Custom");
+	strcpy(constraint[1].option2, constraint->option2);
+	strcpy(constraint[1].choice2, constraint->choice2);
+      }
+      else if (!strcasecmp(constraint->option2, "CustomPageSize") &&
+               (!strcasecmp(constraint->option1, "InputSlot") ||
+	        !strcasecmp(constraint->option1, "ManualFeed")))
+      {
+        ppd->num_consts ++;
+
+	strcpy(constraint[1].option1, constraint->option1);
+	strcpy(constraint[1].choice1, constraint->choice1);
+        strcpy(constraint[1].option2, "PageRegion");
+	strcpy(constraint[1].choice2, "Custom");
+      }
+
      /*
       * Handle CustomFoo option constraints...
       */
@@ -1816,8 +1888,8 @@ ppdOpen2(cups_file_t *fp)		/* I - File to read from */
   cupsLangFree(language);
 
 #ifdef DEBUG
-  if (!feof(fp))
-    printf("Premature EOF at %lu...\n", (unsigned long)ftell(fp));
+  if (!cupsFileEOF(fp))
+    printf("Premature EOF at %lu...\n", (unsigned long)cupsFileTell(fp));
 #endif /* DEBUG */
 
   if (cg->ppd_status != PPD_OK)
@@ -1836,7 +1908,9 @@ ppdOpen2(cups_file_t *fp)		/* I - File to read from */
   * each choice and custom option...
   */
 
-  ppd->options = cupsArrayNew((cups_array_func_t)ppd_compare_options, NULL);
+  ppd->options = cupsArrayNew2((cups_array_func_t)ppd_compare_options, NULL,
+                               (cups_ahash_func_t)ppd_hash_option,
+			       PPD_HASHSIZE);
 
   for (i = ppd->num_groups, group = ppd->groups;
        i > 0;
@@ -1859,6 +1933,20 @@ ppdOpen2(cups_file_t *fp)		/* I - File to read from */
     }
   }
 
+ /*
+  * Sort the constraints...
+  */
+
+  if (ppd->num_consts > 1)
+    qsort(ppd->consts, ppd->num_consts, sizeof(ppd_const_t),
+          (int (*)(const void *, const void *))ppd_compare_consts);
+
+ /*
+  * Create an array to track the marked choices...
+  */
+
+  ppd->marked = cupsArrayNew((cups_array_func_t)ppd_compare_choices, NULL);
+
  /*
   * Return the PPD file structure...
   */
@@ -2152,6 +2240,40 @@ ppd_compare_attrs(ppd_attr_t *a,	/* I - First attribute */
 }
 
 
+/*
+ * 'ppd_compare_choices()' - Compare two choices...
+ */
+
+static int				/* O - Result of comparison */
+ppd_compare_choices(ppd_choice_t *a,	/* I - First choice */
+                    ppd_choice_t *b)	/* I - Second choice */
+{
+  return (strcmp(a->option->keyword, b->option->keyword));
+}
+
+
+/*
+ * 'ppd_compare_consts()' - Compare two constraints.
+ */
+
+static int				/* O - Result of comparison */
+ppd_compare_consts(ppd_const_t *a,	/* I - First constraint */
+                   ppd_const_t *b)	/* I - Second constraint */
+{
+  int	ret;				/* Result of comparison */
+
+
+  if ((ret = strcmp(a->option1, b->option1)) != 0)
+    return (ret);
+  else if ((ret = strcmp(a->choice1, b->choice1)) != 0)
+    return (ret);
+  else if ((ret = strcmp(a->option2, b->option2)) != 0)
+    return (ret);
+  else
+    return (strcmp(a->choice2, b->choice2));
+}
+
+
 /*
  * 'ppd_compare_coptions()' - Compare two custom options.
  */
@@ -2489,6 +2611,24 @@ ppd_get_option(ppd_group_t *group,	/* I - Group */
 }
 
 
+/*
+ * 'ppd_hash_option()' - Generate a hash of the option name...
+ */
+
+static int				/* O - Hash index */
+ppd_hash_option(ppd_option_t *option)	/* I - Option */
+{
+  int		hash = 0;		/* Hash index */
+  const char	*k;			/* Pointer into keyword */
+
+
+  for (hash = option->keyword[0], k = option->keyword + 1; *k;)
+    hash = 33 * hash + *k++;
+
+  return (hash & 511);
+}
+
+
 /*
  * 'ppd_read()' - Read a line from a PPD file, skipping comment lines as
  *                necessary.
@@ -3046,5 +3186,5 @@ ppd_read(cups_file_t    *fp,		/* I - File to read from */
 
 
 /*
- * End of "$Id: ppd.c 6188 2007-01-10 16:23:06Z mike $".
+ * End of "$Id: ppd.c 6942 2007-09-10 22:55:00Z mike $".
  */
