@@ -1,25 +1,18 @@
 /*
- * "$Id: transcode.c 6115 2006-11-15 22:21:18Z mike $"
+ * "$Id: transcode.c 6649 2007-07-11 21:46:42Z mike $"
  *
  *   Transcoding support for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2006 by Easy Software Products.
+ *   Copyright 2007 by Apple Inc.
+ *   Copyright 1997-2007 by Easy Software Products.
  *
- *   These coded instructions, statements, and computer programs are
- *   the property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the
- *   file "LICENSE.txt" which should have been included with this file.
- *   If this file is missing or damaged please contact Easy Software
- *   Products at:
+ *   These coded instructions, statements, and computer programs are the
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ *   which should have been included with this file.  If this file is
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   This file is subject to the Apple OS-Developed Software exception.
  *
  * Contents:
  *
@@ -292,7 +285,7 @@ cupsCharsetToUTF8(
       encoding < 0 || encoding >= CUPS_ENCODING_VBCS_END)
   {
     strlcpy((char *)dest, src, maxout);
-    return (strlen((char *)dest));
+    return ((int)strlen((char *)dest));
   }
 
  /*
@@ -324,7 +317,7 @@ cupsCharsetToUTF8(
 
     *destptr = '\0';
 
-    return (destptr - dest);
+    return ((int)(destptr - dest));
   }
 
  /*
@@ -392,7 +385,7 @@ cupsUTF8ToCharset(
       encoding < 0 || encoding >= CUPS_ENCODING_VBCS_END)
   {
     strlcpy(dest, (char *)src, maxout);
-    return (strlen(dest));
+    return ((int)strlen(dest));
   }
 
  /*
@@ -431,7 +424,7 @@ cupsUTF8ToCharset(
 
     *destptr = '\0';
 
-    return (destptr - dest);
+    return ((int)(destptr - dest));
   }
 
  /*
@@ -968,7 +961,7 @@ conv_utf8_to_vbcs(
 
     if (unichar < 0x80)
     {
-      *dest++ = (cups_vbcs_t)unichar;
+      *dest++ = (cups_sbcs_t)unichar;
       continue;
     }
 
@@ -1435,7 +1428,7 @@ get_vbcs_charmap(
   char		*s;			/* Line parsing pointer */
   char		line[256];		/* Line from charset map file */
   int		i;			/* Loop variable */
-  int		wide;			/* 32-bit legacy char */
+  int		legacy;			/* 32-bit legacy char */
 
 
   DEBUG_printf(("get_vbcs_charmap(encoding=%d, filename=\"%s\")\n",
@@ -1504,8 +1497,8 @@ get_vbcs_charmap(
 
   cupsFileRewind(fp);
 
-  i    = 0;
-  wide = 0;
+  i      = 0;
+  legacy = 0;
 
   while (cupsFileGets(fp, line, sizeof(line)))
   {
@@ -1575,9 +1568,9 @@ get_vbcs_charmap(
       * Save VBCS 32-bit to Unicode mapping in sorted list table...
       */
 
-      if (!wide)
+      if (!legacy)
       {
-	wide            = 1;
+	legacy          = 1;
 	vmap->widecount = (mapcount - i + 1);
 	wide2uni        = (_cups_wide2uni_t *)calloc(vmap->widecount,
 	                                             sizeof(_cups_wide2uni_t));
@@ -1655,5 +1648,5 @@ get_vbcs_charmap(
 
 
 /*
- * End of "$Id: transcode.c 6115 2006-11-15 22:21:18Z mike $"
+ * End of "$Id: transcode.c 6649 2007-07-11 21:46:42Z mike $"
  */
