@@ -1,25 +1,18 @@
 /*
- * "$Id: hpgl-input.c,v 1.11 2002/01/02 17:58:57 mike Exp $"
+ * "$Id: hpgl-input.c 6649 2007-07-11 21:46:42Z mike $"
  *
  *   HP-GL/2 input processing for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1993-2002 by Easy Software Products.
+ *   Copyright 2007 by Apple Inc.
+ *   Copyright 1993-2006 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "LICENSE.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ *   which should have been included with this file.  If this file is
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
- *
- *       Voice: (301) 373-9603
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   This file is subject to the Apple OS-Developed Software exception.
  *
  * Contents:
  *
@@ -33,6 +26,7 @@
 
 #include "hpgltops.h"
 #include <ctype.h>
+#include <cups/i18n.h>
 
 #define MAX_PARAMS 16384
 
@@ -52,7 +46,8 @@ ParseCommand(FILE    *fp,	/* I - File to read from */
 		ch,		/* Current char */
 		done,		/* Non-zero when the current command is read */
 		i;		/* Looping var */
-  char		buf[262144];	/* String buffer */
+  char		buf[262144],	/* String buffer */
+		*bufptr;	/* Pointer into buffer */
   static param_t p[MAX_PARAMS];	/* Parameter buffer */
 
 
@@ -66,7 +61,9 @@ ParseCommand(FILE    *fp,	/* I - File to read from */
         break;
 
     if (ch == EOF)
+    {
       return (-1);
+    }
 
     if (ch == 0x1b)
       switch (getc(fp))
@@ -101,26 +98,61 @@ ParseCommand(FILE    *fp,	/* I - File to read from */
             }
             break;
 
+        case '%' : /* PJL command? */
+            if ((i = getc(fp)) == '-')
+	      if ((i = getc(fp)) == '1')
+	        if ((i = getc(fp)) == '2')
+		{
+		 /*
+		  * Yes, dump everything up to the "ENTER LANGUAGE" line...
+		  */
+
+        	  while (fgets(buf, sizeof(buf), fp) != NULL)
+	            if (strstr(buf, "ENTER") && strstr(buf, "LANGUAGE"))
+		      break;
+		  break;
+		}
+
+            ungetc(i, fp);
+
         default : /* HP RTL/PCL control */
-            while ((i = getc(fp)) != EOF && !isupper(i));
+            while ((i = getc(fp)) != EOF && !isupper(i & 255));
+
+	    if (i == EOF)
+	      return (-1);
             break;
       }
-  } while (ch == 0x1b);
+  } while (ch < ' ');
 
   name[0] = ch;
   name[1] = getc(fp);
   name[2] = '\0';
 
-  if (strcasecmp(name, "LB") == 0)
+  if (name[1] < ' ')
   {
-    for (i = 0; (ch = getc(fp)) != StringTerminator; i ++)
-      buf[i] = ch;
-    buf[i] = '\0';
+   /*
+    * If we get here, then more than likely we are faced with a raw PCL
+    * file which we can't handle - abort!
+    */
+
+    fputs(_("ERROR: Invalid HP-GL/2 command seen, unable to print file!\n"),
+          stderr);
+    return (-1);
+  }
+
+  if (!strcasecmp(name, "LB"))
+  {
+    bufptr = buf;
+    while ((ch = getc(fp)) != StringTerminator)
+      if (bufptr < (buf + sizeof(buf) - 1))
+        *bufptr++ = ch;
+    *bufptr = '\0';
+
     p[num_params].type         = PARAM_STRING;
     p[num_params].value.string = strdup(buf);
     num_params ++;
   }
-  else if (strcasecmp(name, "SM") == 0)
+  else if (!strcasecmp(name, "SM"))
   {
     buf[0] = getc(fp);
     buf[1] = '\0';
@@ -128,7 +160,7 @@ ParseCommand(FILE    *fp,	/* I - File to read from */
     p[num_params].value.string = strdup(buf);
     num_params ++;
   }
-  else if (strcasecmp(name, "DT") == 0)
+  else if (!strcasecmp(name, "DT"))
   {
     if ((buf[0] = getc(fp)) != ';')
     {
@@ -138,13 +170,16 @@ ParseCommand(FILE    *fp,	/* I - File to read from */
       num_params ++;
     }
   }
-  else if (strcasecmp(name, "PE") == 0)
+  else if (!strcasecmp(name, "PE"))
   {
-    for (i = 0; i < (sizeof(buf) - 1); i ++)
-      if ((buf[i] = getc(fp)) == ';')
+    bufptr = buf;
+    while ((ch = getc(fp)) != ';')
+      if (ch == EOF)
         break;
+      else if (bufptr < (buf + sizeof(buf) - 1))
+        *bufptr++ = ch;
+    *bufptr = '\0';
 
-    buf[i] = '\0';
     p[num_params].type         = PARAM_STRING;
     p[num_params].value.string = strdup(buf);
     num_params ++;
@@ -153,6 +188,10 @@ ParseCommand(FILE    *fp,	/* I - File to read from */
   while (!done)
     switch (ch = getc(fp))
     {
+      case EOF :
+          done = 1;
+          break;
+
       case ',' :
       case ' ' :
       case '\n' :
@@ -228,5 +267,5 @@ FreeParameters(int     num_params,	/* I - Number of parameters */
 
 
 /*
- * End of "$Id: hpgl-input.c,v 1.11 2002/01/02 17:58:57 mike Exp $".
+ * End of "$Id: hpgl-input.c 6649 2007-07-11 21:46:42Z mike $".
  */
