@@ -1,25 +1,16 @@
 /*
- * "$Id: image-bmp.c 4741 2005-10-02 04:25:52Z mike $"
+ * "$Id: image-bmp.c 6649 2007-07-11 21:46:42Z mike $"
  *
  *   BMP image routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1993-2005 by Easy Software Products.
+ *   Copyright 2007 by Apple Inc.
+ *   Copyright 1993-2007 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "LICENSE.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ *   which should have been included with this file.  If this file is
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
  *   This file is subject to the Apple OS-Developed Software exception.
  *
@@ -110,7 +101,7 @@ _cupsImageReadBMP(
 
   if (offset < 0)
   {
-    fprintf(stderr, "ERROR: Bad BMP offset %d\n", offset);
+    fprintf(stderr, "DEBUG: Bad BMP offset %d\n", offset);
     fclose(fp);
     return (1);
   }
@@ -135,7 +126,7 @@ _cupsImageReadBMP(
       img->ysize == 0 || img->ysize > CUPS_IMAGE_MAX_HEIGHT ||
       (depth != 1 && depth != 4 && depth != 8 && depth != 24))
   {
-    fprintf(stderr, "ERROR: Bad BMP dimensions %ux%ux%d\n",
+    fprintf(stderr, "DEBUG: Bad BMP dimensions %ux%ux%d\n",
             img->xsize, img->ysize, depth);
     fclose(fp);
     return (1);
@@ -143,14 +134,14 @@ _cupsImageReadBMP(
 
   if (colors_used < 0 || colors_used > 256)
   {
-    fprintf(stderr, "ERROR: Bad BMP colormap size %d\n", colors_used);
+    fprintf(stderr, "DEBUG: Bad BMP colormap size %d\n", colors_used);
     fclose(fp);
     return (1);
   }
 
   if (img->xppi == 0 || img->yppi == 0)
   {
-    fprintf(stderr, "ERROR: Bad BMP resolution %dx%d PPI.\n",
+    fprintf(stderr, "DEBUG: Bad BMP resolution %dx%d PPI.\n",
             img->xppi, img->yppi);
     img->xppi = img->yppi = 128;
   }
@@ -202,10 +193,7 @@ _cupsImageReadBMP(
 
   for (y = img->ysize - 1; y >= 0; y --)
   {
-    if (img->colorspace == CUPS_IMAGE_RGB)
-      ptr = out;
-    else
-      ptr = in;
+    ptr = in;
 
     switch (depth)
     {
@@ -442,37 +430,33 @@ _cupsImageReadBMP(
           break;
     }
 
-    if (img->colorspace == CUPS_IMAGE_RGB)
+    if (saturation != 100 || hue != 0)
+      cupsImageRGBAdjust(in, img->xsize, saturation, hue);
+
+    switch (img->colorspace)
     {
-      if (saturation != 100 || hue != 0)
-	cupsImageRGBAdjust(out, img->xsize, saturation, hue);
-    }
-    else
-    {
-      if (saturation != 100 || hue != 0)
-	cupsImageRGBAdjust(in, img->xsize, saturation, hue);
+      default :
+	  break;
 
-      switch (img->colorspace)
-      {
-        default :
-	    break;
+      case CUPS_IMAGE_WHITE :
+	  cupsImageRGBToWhite(in, out, img->xsize);
+	  break;
 
-	case CUPS_IMAGE_WHITE :
-	    cupsImageRGBToWhite(in, out, img->xsize);
-	    break;
+      case CUPS_IMAGE_RGB :
+	  cupsImageRGBToRGB(in, out, img->xsize);
+	  break;
 
-	case CUPS_IMAGE_BLACK :
-	    cupsImageRGBToBlack(in, out, img->xsize);
-	    break;
+      case CUPS_IMAGE_BLACK :
+	  cupsImageRGBToBlack(in, out, img->xsize);
+	  break;
 
-	case CUPS_IMAGE_CMY :
-	    cupsImageRGBToCMY(in, out, img->xsize);
-	    break;
+      case CUPS_IMAGE_CMY :
+	  cupsImageRGBToCMY(in, out, img->xsize);
+	  break;
 
-	case CUPS_IMAGE_CMYK :
-	    cupsImageRGBToCMYK(in, out, img->xsize);
-	    break;
-      }
+      case CUPS_IMAGE_CMYK :
+	  cupsImageRGBToCMYK(in, out, img->xsize);
+	  break;
     }
 
     if (lut)
@@ -542,5 +526,5 @@ read_long(FILE *fp)               /* I - File to read from */
 
 
 /*
- * End of "$Id: image-bmp.c 4741 2005-10-02 04:25:52Z mike $".
+ * End of "$Id: image-bmp.c 6649 2007-07-11 21:46:42Z mike $".
  */
