@@ -1,41 +1,34 @@
 /*
- * "$Id: image-gif.c,v 1.9 2002/01/02 17:58:58 mike Exp $"
+ * "$Id: image-gif.c 6649 2007-07-11 21:46:42Z mike $"
  *
  *   GIF image routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1993-2002 by Easy Software Products.
+ *   Copyright 2007 by Apple Inc.
+ *   Copyright 1993-2007 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "LICENSE.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ *   which should have been included with this file.  If this file is
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
- *
- *       Voice: (301) 373-9603
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   This file is subject to the Apple OS-Developed Software exception.
  *
  * Contents:
  *
- *   ImageReadGIF()   - Read a GIF image file.
- *   gif_read_cmap()  - Read the colormap from a GIF file...
- *   gif_get_block()  - Read a GIF data block...
- *   gif_get_code()   - Get a LZW code from the file...
- *   gif_read_lzw()   - Read a byte from the LZW stream...
- *   gif_read_image() - Read a GIF image stream...
+ *   _cupsImageReadGIF() - Read a GIF image file.
+ *   gif_get_block()     - Read a GIF data block...
+ *   gif_get_code()      - Get a LZW code from the file...
+ *   gif_read_cmap()     - Read the colormap from a GIF file...
+ *   gif_read_image()    - Read a GIF image stream...
+ *   gif_read_lzw()      - Read a byte from the LZW stream...
  */
 
 /*
  * Include necessary headers...
  */
 
-#include "image.h"
+#include "image-private.h"
 
 
 /*
@@ -45,7 +38,8 @@
 #define GIF_INTERLACE	0x40
 #define GIF_COLORMAP	0x80
 
-typedef ib_t	gif_cmap_t[256][4];
+typedef cups_ib_t	gif_cmap_t[256][4];
+typedef short		gif_table_t[4096];
 
 
 /*
@@ -59,37 +53,45 @@ static int	gif_eof = 0;		/* Did we hit EOF? */
  * Local functions...
  */
 
-static int	gif_read_cmap(FILE *fp, int ncolors, gif_cmap_t cmap,
-		              int *gray);
 static int	gif_get_block(FILE *fp, unsigned char *buffer);
 static int	gif_get_code (FILE *fp, int code_size, int first_time);
-static int	gif_read_lzw(FILE *fp, int first_time, int input_code_size);
-static int	gif_read_image(FILE *fp, image_t *img, gif_cmap_t cmap,
+static int	gif_read_cmap(FILE *fp, int ncolors, gif_cmap_t cmap,
+		              int *gray);
+static int	gif_read_image(FILE *fp, cups_image_t *img, gif_cmap_t cmap,
 		               int interlace);
+static int	gif_read_lzw(FILE *fp, int first_time, int input_code_size);
 
 
 /*
- * 'ImageReadGIF()' - Read a GIF image file.
+ * '_cupsImageReadGIF()' - Read a GIF image file.
  */
 
 int					/* O - Read status */
-ImageReadGIF(image_t    *img,		/* IO - Image */
-             FILE       *fp,		/* I - Image file */
-             int        primary,	/* I - Primary choice for colorspace */
-             int        secondary,	/* I - Secondary choice for colorspace */
-             int        saturation,	/* I - Color saturation (%) */
-             int        hue,		/* I - Color hue (degrees) */
-	     const ib_t *lut)		/* I - Lookup table for gamma/brightness */
+_cupsImageReadGIF(
+    cups_image_t    *img,		/* IO - cupsImage */
+    FILE            *fp,		/* I - cupsImage file */
+    cups_icspace_t  primary,		/* I - Primary choice for colorspace */
+    cups_icspace_t  secondary,		/* I - Secondary choice for colorspace */
+    int             saturation,		/* I - Color saturation (%) */
+    int             hue,		/* I - Color hue (degrees) */
+    const cups_ib_t *lut)		/* I - Lookup table for gamma/brightness */
 {
-  unsigned char	buf[1024];	/* Input buffer */
-  gif_cmap_t	cmap;		/* Colormap */
-  int		i,		/* Looping var */
-		bpp,		/* Bytes per pixel */
-		gray,		/* Grayscale image? */
-		ncolors,	/* Bits per pixel */
-		transparent;	/* Transparent color index */
+  unsigned char	buf[1024];		/* Input buffer */
+  gif_cmap_t	cmap;			/* Colormap */
+  int		i,			/* Looping var */
+		bpp,			/* Bytes per pixel */
+		gray,			/* Grayscale image? */
+		ncolors,		/* Bits per pixel */
+		transparent;		/* Transparent color index */
 
 
+ /*
+  * GIF files are either grayscale or RGB - no CMYK...
+  */
+
+  if (primary == CUPS_IMAGE_RGB_CMYK)
+    primary = CUPS_IMAGE_RGB;
+
  /*
   * Read the header; we already know it is a GIF file...
   */
@@ -99,7 +101,7 @@ ImageReadGIF(image_t    *img,		/* IO - Image */
   img->xsize = (buf[7] << 8) | buf[6];
   img->ysize = (buf[9] << 8) | buf[8];
   ncolors    = 2 << (buf[10] & 0x07);
-  gray       = primary == IMAGE_BLACK || primary == IMAGE_WHITE;
+  gray       = primary == CUPS_IMAGE_BLACK || primary == CUPS_IMAGE_WHITE;
 
   if (buf[10] & GIF_COLORMAP)
     if (gif_read_cmap(fp, ncolors, cmap, &gray))
@@ -130,13 +132,13 @@ ImageReadGIF(image_t    *img,		/* IO - Image */
           while (gif_get_block(fp, buf) != 0);
           break;
 
-      case ',' :	/* Image data */
+      case ',' :	/* cupsImage data */
           fread(buf, 9, 1, fp);
 
           if (buf[8] & GIF_COLORMAP)
           {
             ncolors = 2 << (buf[8] & 0x07);
-            gray = primary == IMAGE_BLACK || primary == IMAGE_WHITE;
+            gray = primary == CUPS_IMAGE_BLACK || primary == CUPS_IMAGE_WHITE;
 
 	    if (gif_read_cmap(fp, ncolors, cmap, &gray))
 	    {
@@ -160,23 +162,24 @@ ImageReadGIF(image_t    *img,		/* IO - Image */
 	  {
 	    switch (secondary)
 	    {
-              case IMAGE_CMYK :
+              case CUPS_IMAGE_CMYK :
         	  for (i = ncolors - 1; i >= 0; i --)
-        	    ImageWhiteToCMYK(cmap[i], cmap[i], 1);
+        	    cupsImageWhiteToCMYK(cmap[i], cmap[i], 1);
         	  break;
-              case IMAGE_CMY :
+              case CUPS_IMAGE_CMY :
         	  for (i = ncolors - 1; i >= 0; i --)
-        	    ImageWhiteToCMY(cmap[i], cmap[i], 1);
+        	    cupsImageWhiteToCMY(cmap[i], cmap[i], 1);
         	  break;
-              case IMAGE_BLACK :
+              case CUPS_IMAGE_BLACK :
         	  for (i = ncolors - 1; i >= 0; i --)
-        	    ImageWhiteToBlack(cmap[i], cmap[i], 1);
+        	    cupsImageWhiteToBlack(cmap[i], cmap[i], 1);
         	  break;
-              case IMAGE_WHITE :
+              case CUPS_IMAGE_WHITE :
         	  break;
-              case IMAGE_RGB :
+              case CUPS_IMAGE_RGB :
+              case CUPS_IMAGE_RGB_CMYK :
         	  for (i = ncolors - 1; i >= 0; i --)
-        	    ImageWhiteToRGB(cmap[i], cmap[i], 1);
+        	    cupsImageWhiteToRGB(cmap[i], cmap[i], 1);
         	  break;
 	    }
 
@@ -186,27 +189,30 @@ ImageReadGIF(image_t    *img,		/* IO - Image */
 	  {
 	    if (hue != 0 || saturation != 100)
               for (i = ncolors - 1; i >= 0; i --)
-        	ImageRGBAdjust(cmap[i], 1, saturation, hue);
+        	cupsImageRGBAdjust(cmap[i], 1, saturation, hue);
 
 	    switch (primary)
 	    {
-              case IMAGE_CMYK :
+              case CUPS_IMAGE_CMYK :
         	  for (i = ncolors - 1; i >= 0; i --)
-        	    ImageRGBToCMYK(cmap[i], cmap[i], 1);
+        	    cupsImageRGBToCMYK(cmap[i], cmap[i], 1);
         	  break;
-              case IMAGE_CMY :
+              case CUPS_IMAGE_CMY :
         	  for (i = ncolors - 1; i >= 0; i --)
-        	    ImageRGBToCMY(cmap[i], cmap[i], 1);
+        	    cupsImageRGBToCMY(cmap[i], cmap[i], 1);
         	  break;
-              case IMAGE_BLACK :
+              case CUPS_IMAGE_BLACK :
         	  for (i = ncolors - 1; i >= 0; i --)
-        	    ImageRGBToBlack(cmap[i], cmap[i], 1);
+        	    cupsImageRGBToBlack(cmap[i], cmap[i], 1);
         	  break;
-              case IMAGE_WHITE :
+              case CUPS_IMAGE_WHITE :
         	  for (i = ncolors - 1; i >= 0; i --)
-        	    ImageRGBToWhite(cmap[i], cmap[i], 1);
+        	    cupsImageRGBToWhite(cmap[i], cmap[i], 1);
         	  break;
-              case IMAGE_RGB :
+              case CUPS_IMAGE_RGB :
+              case CUPS_IMAGE_RGB_CMYK :
+        	  for (i = ncolors - 1; i >= 0; i --)
+        	    cupsImageRGBToRGB(cmap[i], cmap[i], 1);
         	  break;
 	    }
 
@@ -215,15 +221,28 @@ ImageReadGIF(image_t    *img,		/* IO - Image */
 
           if (lut)
 	  {
-	    bpp = ImageGetDepth(img);
+	    bpp = cupsImageGetDepth(img);
 
             for (i = ncolors - 1; i >= 0; i --)
-              ImageLut(cmap[i], bpp, lut);
+              cupsImageLut(cmap[i], bpp, lut);
 	  }
 
           img->xsize = (buf[5] << 8) | buf[4];
           img->ysize = (buf[7] << 8) | buf[6];
 
+         /*
+	  * Check the dimensions of the image; since the dimensions are
+	  * a 16-bit integer we just need to check for 0...
+	  */
+
+          if (img->xsize == 0 || img->ysize == 0)
+	  {
+	    fprintf(stderr, "DEBUG: Bad GIF image dimensions: %dx%d\n",
+	            img->xsize, img->ysize);
+	    fclose(fp);
+	    return (1);
+	  }
+
 	  i = gif_read_image(fp, img, cmap, buf[8] & GIF_INTERLACE);
           fclose(fp);
           return (i);
@@ -232,6 +251,138 @@ ImageReadGIF(image_t    *img,		/* IO - Image */
 }
 
 
+/*
+ * 'gif_get_block()' - Read a GIF data block...
+ */
+
+static int				/* O - Number characters read */
+gif_get_block(FILE          *fp,	/* I - File to read from */
+	      unsigned char *buf)	/* I - Input buffer */
+{
+  int	count;				/* Number of character to read */
+
+
+ /*
+  * Read the count byte followed by the data from the file...
+  */
+
+  if ((count = getc(fp)) == EOF)
+  {
+    gif_eof = 1;
+    return (-1);
+  }
+  else if (count == 0)
+    gif_eof = 1;
+  else if (fread(buf, 1, count, fp) < count)
+  {
+    gif_eof = 1;
+    return (-1);
+  }
+  else
+    gif_eof = 0;
+
+  return (count);
+}
+
+
+/*
+ * 'gif_get_code()' - Get a LZW code from the file...
+ */
+
+static int				/* O - LZW code */
+gif_get_code(FILE *fp,			/* I - File to read from */
+	     int  code_size,		/* I - Size of code in bits */
+	     int  first_time)		/* I - 1 = first time, 0 = not first time */
+{
+  unsigned		i, j,		/* Looping vars */
+			ret;		/* Return value */
+  int			count;		/* Number of bytes read */
+  static unsigned char	buf[280];	/* Input buffer */
+  static unsigned	curbit,		/* Current bit */
+			lastbit,	/* Last bit in buffer */
+			done,		/* Done with this buffer? */
+			last_byte;	/* Last byte in buffer */
+  static const unsigned char bits[8] =	/* Bit masks for codes */
+			{
+			  0x01, 0x02, 0x04, 0x08,
+			  0x10, 0x20, 0x40, 0x80
+			};
+
+
+  if (first_time)
+  {
+   /*
+    * Just initialize the input buffer...
+    */
+
+    curbit    = 0;
+    lastbit   = 0;
+    last_byte = 0;
+    done      = 0;
+
+    return (0);
+  }
+
+  if ((curbit + code_size) >= lastbit)
+  {
+   /*
+    * Don't have enough bits to hold the code...
+    */
+
+    if (done)
+      return (-1);	/* Sorry, no more... */
+
+   /*
+    * Move last two bytes to front of buffer...
+    */
+
+    if (last_byte > 1)
+    {
+      buf[0]    = buf[last_byte - 2];
+      buf[1]    = buf[last_byte - 1];
+      last_byte = 2;
+    }
+    else if (last_byte == 1)
+    {
+      buf[0]    = buf[last_byte - 1];
+      last_byte = 1;
+    }
+
+   /*
+    * Read in another buffer...
+    */
+
+    if ((count = gif_get_block (fp, buf + last_byte)) <= 0)
+    {
+     /*
+      * Whoops, no more data!
+      */
+
+      done = 1;
+      return (-1);
+    }
+
+   /*
+    * Update buffer state...
+    */
+
+    curbit    = (curbit - lastbit) + 8 * last_byte;
+    last_byte += count;
+    lastbit   = last_byte * 8;
+  }
+
+  ret = 0;
+  for (ret = 0, i = curbit + code_size - 1, j = code_size;
+       j > 0;
+       i --, j --)
+    ret = (ret << 1) | ((buf[i / 8] & bits[i & 7]) != 0);
+
+  curbit += code_size;
+
+  return ret;
+}
+
+
 /*
  * 'gif_read_cmap()' - Read the colormap from a GIF file...
  */
@@ -281,125 +432,85 @@ gif_read_cmap(FILE       *fp,		/* I - File to read from */
 
 
 /*
- * 'gif_get_block()' - Read a GIF data block...
+ * 'gif_read_image()' - Read a GIF image stream...
  */
 
-static int				/* O - Number characters read */
-gif_get_block(FILE          *fp,	/* I - File to read from */
-	      unsigned char *buf)	/* I - Input buffer */
+static int				/* I - 0 = success, -1 = failure */
+gif_read_image(FILE         *fp,	/* I - Input file */
+	       cups_image_t *img,	/* I - cupsImage pointer */
+	       gif_cmap_t   cmap,	/* I - Colormap */
+	       int          interlace)	/* I - Non-zero = interlaced image */
 {
-  int	count;				/* Number of character to read */
+  unsigned char		code_size;	/* Code size */
+  cups_ib_t		*pixels,	/* Pixel buffer */
+			*temp;		/* Current pixel */
+  int			xpos,		/* Current X position */
+			ypos,		/* Current Y position */
+			pass;		/* Current pass */
+  int			pixel;		/* Current pixel */
+  int			bpp;		/* Bytes per pixel */
+  static const int	xpasses[4] =	/* X interleaving */
+			{ 8, 8, 4, 2 },
+			ypasses[5] =	/* Y interleaving */
+			{ 0, 4, 2, 1, 999999 };
 
 
- /*
-  * Read the count byte followed by the data from the file...
-  */
+  bpp       = cupsImageGetDepth(img);
+  pixels    = calloc(bpp, img->xsize);
+  xpos      = 0;
+  ypos      = 0;
+  pass      = 0;
+  code_size = getc(fp);
 
-  if ((count = getc(fp)) == EOF)
-  {
-    gif_eof = 1;
+  if (gif_read_lzw(fp, 1, code_size) < 0)
     return (-1);
-  }
-  else if (count == 0)
-    gif_eof = 1;
-  else if (fread(buf, 1, count, fp) < count)
+
+  temp = pixels;
+  while ((pixel = gif_read_lzw(fp, 0, code_size)) >= 0)
   {
-    gif_eof = 1;
-    return (-1);
-  }
-  else
-    gif_eof = 0;
-
-  return (count);
-}
-
-
-/*
- * 'gif_get_code()' - Get a LZW code from the file...
- */
-
-static int			/* O - LZW code */
-gif_get_code(FILE *fp,		/* I - File to read from */
-	     int  code_size,	/* I - Size of code in bits */
-	     int  first_time)	/* I - 1 = first time, 0 = not first time */
-{
-  unsigned		i, j,		/* Looping vars */
-			ret;		/* Return value */
-  int			count;		/* Number of bytes read */
-  static unsigned char	buf[280];	/* Input buffer */
-  static unsigned	curbit,		/* Current bit */
-			lastbit,	/* Last bit in buffer */
-			done,		/* Done with this buffer? */
-			last_byte;	/* Last byte in buffer */
-  static unsigned char	bits[8] =	/* Bit masks for codes */
-			{
-			  0x01, 0x02, 0x04, 0x08,
-			  0x10, 0x20, 0x40, 0x80
-			};
-
-
-  if (first_time)
-  {
-   /*
-    * Just initialize the input buffer...
-    */
-
-    curbit  = 0;
-    lastbit = 0;
-    done    = 0;
-
-    return (0);
-  }
-
-
-  if ((curbit + code_size) >= lastbit)
-  {
-   /*
-    * Don't have enough bits to hold the code...
-    */
-
-    if (done)
-      return (-1);	/* Sorry, no more... */
-
-   /*
-    * Move last two bytes to front of buffer...
-    */
-
-    buf[0] = buf[last_byte - 2];
-    buf[1] = buf[last_byte - 1];
-
-   /*
-    * Read in another buffer...
-    */
-
-    if ((count = gif_get_block (fp, buf + 2)) <= 0)
+    switch (bpp)
     {
-     /*
-      * Whoops, no more data!
-      */
-
-      done = 1;
-      return (-1);
+      case 4 :
+          temp[3] = cmap[pixel][3];
+      case 3 :
+          temp[2] = cmap[pixel][2];
+      case 2 :
+          temp[1] = cmap[pixel][1];
+      default :
+          temp[0] = cmap[pixel][0];
     }
 
-   /*
-    * Update buffer state...
-    */
+    xpos ++;
+    temp += bpp;
+    if (xpos == img->xsize)
+    {
+      _cupsImagePutRow(img, 0, ypos, img->xsize, pixels);
 
-    last_byte = 2 + count;
-    curbit    = (curbit - lastbit) + 16;
-    lastbit   = last_byte * 8;
+      xpos = 0;
+      temp = pixels;
+
+      if (interlace)
+      {
+        ypos += xpasses[pass];
+
+        if (ypos >= img->ysize)
+	{
+	  pass ++;
+
+          ypos = ypasses[pass];
+	}
+      }
+      else
+	ypos ++;
+    }
+
+    if (ypos >= img->ysize)
+      break;
   }
 
-  ret = 0;
-  for (ret = 0, i = curbit + code_size - 1, j = code_size;
-       j > 0;
-       i --, j --)
-    ret = (ret << 1) | ((buf[i / 8] & bits[i & 7]) != 0);
+  free(pixels);
 
-  curbit += code_size;
-
-  return ret;
+  return (0);
 }
 
 
@@ -412,21 +523,21 @@ gif_read_lzw(FILE *fp,			/* I - File to read from */
 	     int  first_time,		/* I - 1 = first time, 0 = not first time */
  	     int  input_code_size)	/* I - Code size in bits */
 {
-  int		i,			/* Looping var */
-		code,			/* Current code */
-		incode;			/* Input code */
-  static short	fresh = 0,		/* 1 = empty buffers */
-		code_size,		/* Current code size */
-		set_code_size,		/* Initial code size set */
-		max_code,		/* Maximum code used */
-		max_code_size,		/* Maximum code size */
-		firstcode,		/* First code read */
-		oldcode,		/* Last code read */
-		clear_code,		/* Clear code for LZW input */
-		end_code,		/* End code for LZW input */
-		table[2][4096],		/* String table */
-		stack[8192],		/* Output stack */
-		*sp;			/* Current stack pointer */
+  int			i,		/* Looping var */
+			code,		/* Current code */
+			incode;		/* Input code */
+  static short		fresh = 0,	/* 1 = empty buffers */
+			code_size,	/* Current code size */
+			set_code_size,	/* Initial code size set */
+			max_code,	/* Maximum code used */
+			max_code_size,	/* Maximum code size */
+			firstcode,	/* First code read */
+			oldcode,	/* Last code read */
+			clear_code,	/* Clear code for LZW input */
+			end_code,	/* End code for LZW input */
+			*stack = NULL,	/* Output stack */
+			*sp;		/* Current stack pointer */
+  static gif_table_t	*table = NULL;	/* String table */
 
 
   if (first_time)
@@ -442,6 +553,22 @@ gif_read_lzw(FILE *fp,			/* I - File to read from */
     max_code_size = 2 * clear_code;
     max_code      = clear_code + 2;
 
+   /*
+    * Allocate memory for buffers...
+    */
+
+    if (table == NULL)
+      table = calloc(2, sizeof(gif_table_t));
+
+    if (table == NULL)
+      return (-1);
+
+    if (stack == NULL)
+      stack = calloc(8192, sizeof(short));
+
+    if (stack == NULL)
+      return (-1);
+
    /*
     * Initialize input buffers...
     */
@@ -559,86 +686,5 @@ gif_read_lzw(FILE *fp,			/* I - File to read from */
 
 
 /*
- * 'gif_read_image()' - Read a GIF image stream...
- */
-
-static int				/* I - 0 = success, -1 = failure */
-gif_read_image(FILE       *fp,		/* I - Input file */
-	       image_t    *img,		/* I - Image pointer */
-	       gif_cmap_t cmap,		/* I - Colormap */
-	       int        interlace)	/* I - Non-zero = interlaced image */
-{
-  unsigned char	code_size;		/* Code size */
-  ib_t		*pixels,		/* Pixel buffer */
-		*temp;			/* Current pixel */
-  int		xpos,			/* Current X position */
-		ypos,			/* Current Y position */
-		pass;			/* Current pass */
-  int		pixel;			/* Current pixel */
-  int		bpp;			/* Bytes per pixel */
-  static int	xpasses[4] = { 8, 8, 4, 2 },
-		ypasses[5] = { 0, 4, 2, 1, 999999 };
-
-
-  bpp       = ImageGetDepth(img);
-  pixels    = calloc(bpp, img->xsize);
-  xpos      = 0;
-  ypos      = 0;
-  pass      = 0;
-  code_size = getc(fp);
-
-  if (gif_read_lzw(fp, 1, code_size) < 0)
-    return (-1);
-
-  temp = pixels;
-  while ((pixel = gif_read_lzw(fp, 0, code_size)) >= 0)
-  {
-    switch (bpp)
-    {
-      case 4 :
-          temp[3] = cmap[pixel][3];
-      case 3 :
-          temp[2] = cmap[pixel][2];
-      case 2 :
-          temp[1] = cmap[pixel][1];
-      default :
-          temp[0] = cmap[pixel][0];
-    }
-
-    xpos ++;
-    temp += bpp;
-    if (xpos == img->xsize)
-    {
-      ImagePutRow(img, 0, ypos, img->xsize, pixels);
-
-      xpos = 0;
-      temp = pixels;
-
-      if (interlace)
-      {
-        ypos += xpasses[pass];
-
-        if (ypos >= img->ysize)
-	{
-	  pass ++;
-
-          ypos = ypasses[pass];
-	}
-      }
-      else
-	ypos ++;
-    }
-
-    if (ypos >= img->ysize)
-      break;
-  }
-
-  free(pixels);
-
-  return (0);
-}
-
-
-/*
- * End of "$Id: image-gif.c,v 1.9 2002/01/02 17:58:58 mike Exp $".
+ * End of "$Id: image-gif.c 6649 2007-07-11 21:46:42Z mike $".
  */
