@@ -1,25 +1,16 @@
 /*
- * "$Id: image-pix.c 5509 2006-05-11 11:41:36Z mike $"
+ * "$Id: image-pix.c 6649 2007-07-11 21:46:42Z mike $"
  *
  *   Alias PIX image routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1993-2006 by Easy Software Products.
+ *   Copyright 2007 by Apple Inc.
+ *   Copyright 1993-2007 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "LICENSE.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ *   which should have been included with this file.  If this file is
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
  *   This file is subject to the Apple OS-Developed Software exception.
  *
@@ -88,7 +79,7 @@ _cupsImageReadPIX(
   if (width <= 0 || height <= 0 ||
       (depth != 8 && depth != 24))
   {
-    fprintf(stderr, "ERROR: Bad PIX image dimensions %dx%dx%d\n",
+    fprintf(stderr, "DEBUG: Bad PIX image dimensions %dx%dx%d\n",
             width, height, depth);
     fclose(fp);
     return (1);
@@ -232,5 +223,5 @@ read_short(FILE *fp)			/* I - File to read from */
 
 
 /*
- * End of "$Id: image-pix.c 5509 2006-05-11 11:41:36Z mike $".
+ * End of "$Id: image-pix.c 6649 2007-07-11 21:46:42Z mike $".
  */
