@@ -1,25 +1,16 @@
 /*
- * "$Id: image-pnm.c 6149 2006-12-06 20:25:42Z mike $"
+ * "$Id: image-pnm.c 6649 2007-07-11 21:46:42Z mike $"
  *
  *   Portable Any Map file routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1993-2006 by Easy Software Products.
+ *   Copyright 2007 by Apple Inc.
+ *   Copyright 1993-2007 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "LICENSE.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ *   which should have been included with this file.  If this file is
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
  *   This file is subject to the Apple OS-Developed Software exception.
  *
@@ -135,7 +126,7 @@ _cupsImageReadPNM(
   if (img->xsize == 0 || img->xsize > CUPS_IMAGE_MAX_WIDTH ||
       img->ysize == 0 || img->ysize > CUPS_IMAGE_MAX_HEIGHT)
   {
-    fprintf(stderr, "ERROR: Bad PNM dimensions %dx%d!\n",
+    fprintf(stderr, "DEBUG: Bad PNM dimensions %dx%d!\n",
             img->xsize, img->ysize);
     fclose(fp);
     return (1);
@@ -143,7 +134,7 @@ _cupsImageReadPNM(
 
   if (maxval == 0)
   {
-    fprintf(stderr, "ERROR: Bad PNM max value %d!\n", maxval);
+    fprintf(stderr, "DEBUG: Bad PNM max value %d!\n", maxval);
     fclose(fp);
     return (1);
   }
@@ -301,5 +292,5 @@ _cupsImageReadPNM(
 
 
 /*
- * End of "$Id: image-pnm.c 6149 2006-12-06 20:25:42Z mike $".
+ * End of "$Id: image-pnm.c 6649 2007-07-11 21:46:42Z mike $".
  */
