@@ -1,25 +1,16 @@
 /*
- * "$Id: interpret.c 6282 2007-02-14 16:33:54Z mike $"
+ * "$Id: interpret.c 6649 2007-07-11 21:46:42Z mike $"
  *
  *   PPD command interpreter for the Common UNIX Printing System (CUPS).
  *
+ *   Copyright 2007 by Apple Inc.
  *   Copyright 1993-2007 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "LICENSE.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ *   which should have been included with this file.  If this file is
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
  *   This file is subject to the Apple OS-Developed Software exception.
  *
@@ -31,6 +22,9 @@
  *   cleartomark_stack()      - Clear to the last mark ([) on the stack.
  *   copy_stack()             - Copy the top N stack objects.
  *   delete_stack()           - Free memory used by a stack.
+ *   error_object()           - Add an object's value to the current error
+ *                              message.
+ *   error_stack()            - Add a stack to the current error message.
  *   index_stack()            - Copy the Nth value on the stack.
  *   new_stack()              - Create a new stack.
  *   pop_stock()              - Pop the top object off the stack.
@@ -107,6 +101,8 @@ typedef struct
 static int		cleartomark_stack(_cups_ps_stack_t *st);
 static int		copy_stack(_cups_ps_stack_t *st, int count);
 static void		delete_stack(_cups_ps_stack_t *st);
+static void		error_object(_cups_ps_obj_t *obj);
+static void		error_stack(_cups_ps_stack_t *st, const char *title);
 static _cups_ps_obj_t	*index_stack(_cups_ps_stack_t *st, int n);
 static _cups_ps_stack_t	*new_stack(void);
 static _cups_ps_obj_t	*pop_stack(_cups_ps_stack_t *st);
@@ -167,8 +163,13 @@ cupsRasterInterpretPPD(
   * Range check input...
   */
 
+  _cupsRasterClearError();
+
   if (!h)
+  {
+    _cupsRasterAddError("Page header cannot be NULL!\n");
     return (-1);
+  }
 
  /*
   * Reset the page header to the defaults...
@@ -248,7 +249,7 @@ cupsRasterInterpretPPD(
   {
     float sc = atof(val);
 
-    if (sc >= 0.9 && sc <= 1.1)
+    if (sc >= 0.5 && sc <= 2.0)
       h->cupsBorderlessScalingFactor = sc;
   }
 
@@ -304,7 +305,10 @@ cupsRasterInterpretPPD(
   */
 
   if (func && (*func)(h, preferred_bits))
+  {
+    _cupsRasterAddError("Page header callback returned error.\n");
     return (-1);
+  }
 
  /*
   * Check parameters...
@@ -317,7 +321,10 @@ cupsRasterInterpretPPD(
        h->cupsBitsPerColor != 16) ||
       h->cupsBorderlessScalingFactor < 0.5 ||
       h->cupsBorderlessScalingFactor > 2.0)
+  {
+    _cupsRasterAddError("Page header uses unsupported values.\n");
     return (-1);
+  }
 
  /*
   * Compute the bitmap parameters...
@@ -431,10 +438,14 @@ _cupsRasterExecPS(
   */
 
   if ((codecopy = strdup(code)) == NULL)
+  {
+    _cupsRasterAddError("Unable to duplicate code string.\n");
     return (-1);
+  }
 
   if ((st = new_stack()) == NULL)
   {
+    _cupsRasterAddError("Unable to create stack.\n");
     free(codecopy);
     return (-1);
   }
@@ -462,7 +473,8 @@ _cupsRasterExecPS(
       case CUPS_PS_CLEARTOMARK :
           pop_stack(st);
 
-	  cleartomark_stack(st);
+	  if (cleartomark_stack(st))
+	    _cupsRasterAddError("cleartomark: Stack underflow!\n");
 
 #ifdef DEBUG
           fputs("    dup: ", stdout);
@@ -554,6 +566,7 @@ _cupsRasterExecPS(
 	  break;
 
       case CUPS_PS_OTHER :
+          _cupsRasterAddError("Unknown operator \"%s\"!\n", obj->value.other);
           DEBUG_printf(("    Unknown operator \"%s\"!\n", obj->value.other));
           break;
     }
@@ -570,6 +583,8 @@ _cupsRasterExecPS(
 
   if (st->num_objs > 0)
   {
+    error_stack(st, "Stack not empty:");
+
 #ifdef DEBUG
     fputs("    Stack not empty:", stdout);
     DEBUG_stack(st);
@@ -652,6 +667,122 @@ delete_stack(_cups_ps_stack_t *st)	/* I - Stack */
 }
 
 
+/*
+ * 'error_object()' - Add an object's value to the current error message.
+ */
+
+static void
+error_object(_cups_ps_obj_t *obj)	/* I - Object to add */
+{
+  switch (obj->type)
+  {
+    case CUPS_PS_NAME :
+	_cupsRasterAddError(" /%s", obj->value.name);
+	break;
+
+    case CUPS_PS_NUMBER :
+	_cupsRasterAddError(" %g", obj->value.number);
+	break;
+
+    case CUPS_PS_STRING :
+	_cupsRasterAddError(" (%s)", obj->value.string);
+	break;
+
+    case CUPS_PS_BOOLEAN :
+	if (obj->value.boolean)
+	  _cupsRasterAddError(" true");
+	else
+	  _cupsRasterAddError(" false");
+	break;
+
+    case CUPS_PS_NULL :
+	_cupsRasterAddError(" null");
+	break;
+
+    case CUPS_PS_START_ARRAY :
+	_cupsRasterAddError(" [");
+	break;
+
+    case CUPS_PS_END_ARRAY :
+	_cupsRasterAddError(" ]");
+	break;
+
+    case CUPS_PS_START_DICT :
+	_cupsRasterAddError(" <<");
+	break;
+
+    case CUPS_PS_END_DICT :
+	_cupsRasterAddError(" >>");
+	break;
+
+    case CUPS_PS_START_PROC :
+	_cupsRasterAddError(" {");
+	break;
+
+    case CUPS_PS_END_PROC :
+	_cupsRasterAddError(" }");
+	break;
+
+    case CUPS_PS_COPY :
+	_cupsRasterAddError(" --copy--");
+        break;
+
+    case CUPS_PS_CLEARTOMARK :
+	_cupsRasterAddError(" --cleartomark--");
+        break;
+
+    case CUPS_PS_DUP :
+	_cupsRasterAddError(" --dup--");
+        break;
+
+    case CUPS_PS_INDEX :
+	_cupsRasterAddError(" --index--");
+        break;
+
+    case CUPS_PS_POP :
+	_cupsRasterAddError(" --pop--");
+        break;
+
+    case CUPS_PS_ROLL :
+	_cupsRasterAddError(" --roll--");
+        break;
+
+    case CUPS_PS_SETPAGEDEVICE :
+	_cupsRasterAddError(" --setpagedevice--");
+        break;
+
+    case CUPS_PS_STOPPED :
+	_cupsRasterAddError(" --stopped--");
+        break;
+
+    case CUPS_PS_OTHER :
+	_cupsRasterAddError(" --%s--", obj->value.other);
+	break;
+  }
+}
+
+
+/*
+ * 'error_stack()' - Add a stack to the current error message...
+ */
+
+static void
+error_stack(_cups_ps_stack_t *st,	/* I - Stack */
+            const char       *title)	/* I - Title string */
+{
+  int			c;		/* Looping var */
+  _cups_ps_obj_t	*obj;		/* Current object on stack */
+
+
+  _cupsRasterAddError(title);
+
+  for (obj = st->objs, c = st->num_objs; c > 0; c --, obj ++)
+    error_object(obj);
+
+  _cupsRasterAddError("\n");
+}
+
+
 /*
  * 'index_stack()' - Copy the Nth value on the stack.
  */
@@ -1275,9 +1406,19 @@ setpagedevice(
       h->LeadingEdge = (unsigned)obj->value.number;
     else if (!strcmp(name, "ManualFeed") && obj->type == CUPS_PS_BOOLEAN)
       h->ManualFeed = (unsigned)obj->value.boolean;
-    else if ((!strcmp(name, "cupsMediaPosition") || /* Compatibility */
+    else if ((!strcmp(name, "cupsMediaPosition") ||
               !strcmp(name, "MediaPosition")) && obj->type == CUPS_PS_NUMBER)
+    {
+     /*
+      * cupsMediaPosition is supported for backwards compatibility only.
+      * We added it back in the Ghostscript 5.50 days to work around a
+      * bug in Ghostscript WRT handling of MediaPosition and setpagedevice.
+      *
+      * All new development should set MediaPosition...
+      */
+
       h->MediaPosition = (unsigned)obj->value.number;
+    }
     else if (!strcmp(name, "MediaWeight") && obj->type == CUPS_PS_NUMBER)
       h->MediaWeight = (unsigned)obj->value.number;
     else if (!strcmp(name, "MirrorPrint") && obj->type == CUPS_PS_BOOLEAN)
@@ -1500,5 +1641,5 @@ DEBUG_stack(_cups_ps_stack_t *st)	/* I - Stack */
 
 
 /*
- * End of "$Id: interpret.c 6282 2007-02-14 16:33:54Z mike $".
+ * End of "$Id: interpret.c 6649 2007-07-11 21:46:42Z mike $".
  */
