@@ -1,25 +1,16 @@
 /*
- * "$Id: rastertolabel.c 5703 2006-06-29 18:12:04Z mike $"
+ * "$Id: rastertolabel.c 6820 2007-08-20 21:15:28Z mike $"
  *
  *   Label printer filter for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 2001-2006 by Easy Software Products.
+ *   Copyright 2007 by Apple Inc.
+ *   Copyright 2001-2007 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "LICENSE.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ *   which should have been included with this file.  If this file is
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
  *   This file is subject to the Apple OS-Developed Software exception.
  *
@@ -30,6 +21,7 @@
  *   EndPage()      - Finish a page of graphics.
  *   CancelJob()    - Cancel the current job...
  *   OutputLine()   - Output a line of graphics.
+ *   PCLCompress()  - Output a PCL (mode 3) compressed line.
  *   ZPLCompress()  - Output a run-length compression sequence.
  *   main()         - Main entry and processing of driver.
  */
@@ -40,6 +32,7 @@
 
 #include <cups/cups.h>
 #include <cups/string.h>
+#include <cups/i18n.h>
 #include "raster.h"
 #include <stdlib.h>
 #include <unistd.h>
@@ -48,16 +41,20 @@
 
 
 /*
- * This driver filter currently supports Dymo and Zebra label printers.
+ * This driver filter currently supports Dymo, Intellitech, and Zebra
+ * label printers.
  *
  * The Dymo portion of the driver has been tested with the 300, 330,
- * and 330 Turbo label printers; it may also work with older models.
+ * and 330 Turbo label printers; it may also work with other models.
  * The Dymo printers support printing at 136, 203, and 300 DPI.
  *
- * The Zebra portion of the driver has been tested with the LP-2844Z label
- * printer; it may also work with other models.  The driver supports EPL
- * line mode, EPL page mode, ZPL, and CPCL as defined in Zebra's on-line
- * developer documentation.
+ * The Intellitech portion of the driver has been tested with the
+ * Intellibar 408, 412, and 808 and supports their PCL variant.
+ *
+ * The Zebra portion of the driver has been tested with the LP-2844,
+ * LP-2844Z, QL-320, and QL-420 label printers; it may also work with
+ * other models.  The driver supports EPL line mode, EPL page mode,
+ * ZPL, and CPCL as defined in Zebra's on-line developer documentation.
  */
 
 /*
@@ -71,13 +68,15 @@
 #define ZEBRA_ZPL	0x12		/* Zebra ZPL-based printers */
 #define ZEBRA_CPCL	0x13		/* Zebra CPCL-based printers */
 
+#define INTELLITECH_PCL	0x20		/* Intellitech PCL-based printers */
+
 
 /*
  * Globals...
  */
 
 unsigned char	*Buffer;		/* Output buffer */
-char		*CompBuffer;		/* Compression buffer */
+unsigned char	*CompBuffer;		/* Compression buffer */
 unsigned char	*LastBuffer;		/* Last buffer */
 int		LastSet;		/* Number of repeat characters */
 int		ModelNumber,		/* cupsModelNumber attribute */
@@ -95,6 +94,7 @@ void	StartPage(ppd_file_t *ppd, cups_page_header_t *header);
 void	EndPage(ppd_file_t *ppd, cups_page_header_t *header);
 void	CancelJob(int sig);
 void	OutputLine(ppd_file_t *ppd, cups_page_header_t *header, int y);
+void	PCLCompress(unsigned char *line, int length);
 void	ZPLCompress(char repeat_char, int repeat_count);
 
 
@@ -147,6 +147,15 @@ Setup(ppd_file_t *ppd)			/* I - PPD file */
 
     case ZEBRA_CPCL :
         break;
+
+    case INTELLITECH_PCL :
+       /*
+	* Send a PCL reset sequence.
+	*/
+
+	putchar(0x1b);
+	putchar('E');
+        break;
   }
 }
 
@@ -213,6 +222,9 @@ StartPage(ppd_file_t         *ppd,	/* I - PPD file */
   fprintf(stderr, "DEBUG: cupsColorOrder = %d\n", header->cupsColorOrder);
   fprintf(stderr, "DEBUG: cupsColorSpace = %d\n", header->cupsColorSpace);
   fprintf(stderr, "DEBUG: cupsCompression = %d\n", header->cupsCompression);
+  fprintf(stderr, "DEBUG: cupsRowCount = %d\n", header->cupsRowCount);
+  fprintf(stderr, "DEBUG: cupsRowFeed = %d\n", header->cupsRowFeed);
+  fprintf(stderr, "DEBUG: cupsRowStep = %d\n", header->cupsRowStep);
 
  /*
   * Register a signal handler to eject the current page if the
@@ -316,7 +328,7 @@ StartPage(ppd_file_t         *ppd,	/* I - PPD file */
         * Set label size...
 	*/
 
-        printf("q%d\n", header->cupsWidth);
+        printf("q%d\n", (header->cupsWidth + 7) & ~7);
         break;
 
     case ZEBRA_ZPL :
@@ -355,6 +367,120 @@ StartPage(ppd_file_t         *ppd,	/* I - PPD file */
 	printf("PAGE-WIDTH %d\r\n", header->cupsWidth);
 	printf("PAGE-HEIGHT %d\r\n", header->cupsWidth);
         break;
+
+    case INTELLITECH_PCL :
+       /*
+        * Set the media size...
+	*/
+
+	printf("\033&l6D\033&k12H");	/* Set 6 LPI, 10 CPI */
+	printf("\033&l0O");		/* Set portrait orientation */
+
+	switch (header->PageSize[1])
+	{
+	  case 540 : /* Monarch Envelope */
+              printf("\033&l80A");	/* Set page size */
+	      break;
+
+	  case 624 : /* DL Envelope */
+              printf("\033&l90A");	/* Set page size */
+	      break;
+
+	  case 649 : /* C5 Envelope */
+              printf("\033&l91A");	/* Set page size */
+	      break;
+
+	  case 684 : /* COM-10 Envelope */
+              printf("\033&l81A");	/* Set page size */
+	      break;
+
+	  case 756 : /* Executive */
+              printf("\033&l1A");	/* Set page size */
+	      break;
+
+	  case 792 : /* Letter */
+              printf("\033&l2A");	/* Set page size */
+	      break;
+
+	  case 842 : /* A4 */
+              printf("\033&l26A");	/* Set page size */
+	      break;
+
+	  case 1008 : /* Legal */
+              printf("\033&l3A");	/* Set page size */
+	      break;
+
+          default : /* Custom size */
+	      printf("\033!f%dZ", header->PageSize[1] * 300 / 72);
+	      break;
+	}
+
+	printf("\033&l%dP",		/* Set page length */
+               header->PageSize[1] / 12);
+	printf("\033&l0E");		/* Set top margin to 0 */
+        printf("\033&l%dX", header->NumCopies);
+					/* Set number copies */
+        printf("\033&l0L");		/* Turn off perforation skip */
+
+       /*
+        * Print settings...
+	*/
+
+	if (Page == 1)
+	{
+          if (header->cupsRowFeed)	/* inPrintRate */
+	    printf("\033!p%dS", header->cupsRowFeed);
+
+          if (header->cupsCompression != ~0)
+	  				/* inPrintDensity */
+	    printf("\033&d%dA", 30 * header->cupsCompression / 100 - 15);
+
+	  if ((choice = ppdFindMarkedChoice(ppd, "inPrintMode")) != NULL)
+	  {
+	    if (!strcmp(choice->choice, "Standard"))
+	      fputs("\033!p0M", stdout);
+	    else if (!strcmp(choice->choice, "Tear"))
+	    {
+	      fputs("\033!p1M", stdout);
+
+              if (header->cupsRowCount)	/* inTearInterval */
+		printf("\033!n%dT", header->cupsRowCount);
+            }
+	    else
+	    {
+	      fputs("\033!p2M", stdout);
+
+              if (header->cupsRowStep)	/* inCutInterval */
+		printf("\033!n%dC", header->cupsRowStep);
+            }
+	  }
+        }
+
+       /*
+	* Setup graphics...
+	*/
+
+	printf("\033*t%dR", header->HWResolution[0]);
+					/* Set resolution */
+
+	printf("\033*r%dS", header->cupsWidth);
+					/* Set width */
+	printf("\033*r%dT", header->cupsHeight);
+					/* Set height */
+
+	printf("\033&a0H");		/* Set horizontal position */
+	printf("\033&a0V");		/* Set vertical position */
+        printf("\033*r1A");		/* Start graphics */
+        printf("\033*b3M");		/* Set compression */
+
+       /*
+        * Allocate compression buffers...
+	*/
+
+	CompBuffer = malloc(2 * header->cupsBytesPerLine + 1);
+	LastBuffer = malloc(header->cupsBytesPerLine);
+	LastSet    = 0;
+        break;
   }
 
  /*
@@ -534,7 +660,8 @@ EndPage(ppd_file_t *ppd,		/* I - PPD file */
         * End the label and eject...
 	*/
 
-        puts("^XZ");
+        puts("^IDR:CUPS.GRF^FS");
+	puts("^XZ");
 
        /*
         * Free compression buffers...
@@ -596,6 +723,11 @@ EndPage(ppd_file_t *ppd,		/* I - PPD file */
 
 	puts("PRINT\r");
 	break;
+
+    case INTELLITECH_PCL :
+        printf("\033*rB");		/* End GFX */
+        printf("\014");			/* Eject current page */
+        break;
   }
 
   fflush(stdout);
@@ -750,7 +882,7 @@ OutputLine(ppd_file_t         *ppd,	/* I - PPD file */
         * Run-length compress the graphics...
 	*/
 
-	for (compptr = CompBuffer, repeat_char = CompBuffer[0], repeat_count = 1;
+	for (compptr = CompBuffer + 1, repeat_char = CompBuffer[0], repeat_count = 1;
 	     *compptr;
 	     compptr ++)
 	  if (*compptr == repeat_char)
@@ -769,13 +901,19 @@ OutputLine(ppd_file_t         *ppd,	/* I - PPD file */
 	  */
 
 	  if (repeat_count & 1)
+	  {
+	    repeat_count --;
 	    putchar('0');
+	  }
 
-	  putchar(',');
+          if (repeat_count > 0)
+	    putchar(',');
 	}
 	else
 	  ZPLCompress(repeat_char, repeat_count);
 
+	fflush(stdout);
+
        /*
         * Save this line for the next round...
 	*/
@@ -793,10 +931,163 @@ OutputLine(ppd_file_t         *ppd,	/* I - PPD file */
 	  fflush(stdout);
 	}
 	break;
+
+    case INTELLITECH_PCL :
+	if (Buffer[0] ||
+            memcmp(Buffer, Buffer + 1, header->cupsBytesPerLine - 1))
+        {
+	  if (Feed)
+	  {
+	    printf("\033*b%dY", Feed);
+	    Feed    = 0;
+	    LastSet = 0;
+	  }
+
+          PCLCompress(Buffer, header->cupsBytesPerLine);
+	}
+	else
+	  Feed ++;
+        break;
   }
 }
 
 
+/*
+ * 'PCLCompress()' - Output a PCL (mode 3) compressed line.
+ */
+
+void
+PCLCompress(unsigned char *line,	/* I - Line to compress */
+            int           length)	/* I - Length of line */
+{
+  unsigned char	*line_ptr,		/* Current byte pointer */
+        	*line_end,		/* End-of-line byte pointer */
+        	*comp_ptr,		/* Pointer into compression buffer */
+        	*start,			/* Start of compression sequence */
+		*seed;			/* Seed buffer pointer */
+  int           count,			/* Count of bytes for output */
+		offset;			/* Offset of bytes for output */
+
+
+ /*
+  * Do delta-row compression...
+  */
+
+  line_ptr = line;
+  line_end = line + length;
+
+  comp_ptr = CompBuffer;
+  seed     = LastBuffer;
+
+  while (line_ptr < line_end)
+  {
+   /*
+    * Find the next non-matching sequence...
+    */
+
+    start = line_ptr;
+
+    if (!LastSet)
+    {
+     /*
+      * The seed buffer is invalid, so do the next 8 bytes, max...
+      */
+
+      offset = 0;
+
+      if ((count = line_end - line_ptr) > 8)
+	count = 8;
+
+      line_ptr += count;
+    }
+    else
+    {
+     /*
+      * The seed buffer is valid, so compare against it...
+      */
+
+      while (*line_ptr == *seed &&
+             line_ptr < line_end)
+      {
+        line_ptr ++;
+        seed ++;
+      }
+
+      if (line_ptr == line_end)
+        break;
+
+      offset = line_ptr - start;
+
+     /*
+      * Find up to 8 non-matching bytes...
+      */
+
+      start = line_ptr;
+      count = 0;
+      while (*line_ptr != *seed &&
+             line_ptr < line_end &&
+             count < 8)
+      {
+        line_ptr ++;
+        seed ++;
+        count ++;
+      }
+    }
+
+   /*
+    * Place mode 3 compression data in the buffer; see HP manuals
+    * for details...
+    */
+
+    if (offset >= 31)
+    {
+     /*
+      * Output multi-byte offset...
+      */
+
+      *comp_ptr++ = ((count - 1) << 5) | 31;
+
+      offset -= 31;
+      while (offset >= 255)
+      {
+        *comp_ptr++ = 255;
+        offset    -= 255;
+      }
+
+      *comp_ptr++ = offset;
+    }
+    else
+    {
+     /*
+      * Output single-byte offset...
+      */
+
+      *comp_ptr++ = ((count - 1) << 5) | offset;
+    }
+
+    memcpy(comp_ptr, start, count);
+    comp_ptr += count;
+  }
+
+  line_ptr = CompBuffer;
+  line_end = comp_ptr;
+
+ /*
+  * Set the length of the data and write it...
+  */
+
+  printf("\033*b%dW", (int)(comp_ptr - CompBuffer));
+  fwrite(CompBuffer, comp_ptr - CompBuffer, 1, stdout);
+
+ /*
+  * Save this line as a "seed" buffer for the next...
+  */
+
+  memcpy(LastBuffer, line, length);
+  LastSet = 1;
+}
+
+
 /*
  * 'ZPLCompress()' - Output a run-length compression sequence.
  */
@@ -878,7 +1169,8 @@ main(int  argc,				/* I - Number of command-line arguments */
     * and return.
     */
 
-    fputs("ERROR: rastertodymo job-id user title copies options [file]\n", stderr);
+    fprintf(stderr, _("Usage: %s job-id user title copies options [file]\n"),
+            argv[0]);
     return (1);
   }
 
@@ -952,7 +1244,7 @@ main(int  argc,				/* I - Number of command-line arguments */
       */
 
       if ((y & 15) == 0)
-        fprintf(stderr, "INFO: Printing page %d, %d%% complete...\n", Page,
+        fprintf(stderr, _("INFO: Printing page %d, %d%% complete...\n"), Page,
 	        100 * y / header.cupsHeight);
 
      /*
@@ -999,14 +1291,14 @@ main(int  argc,				/* I - Number of command-line arguments */
   */
 
   if (Page == 0)
-    fputs("ERROR: No pages found!\n", stderr);
+    fputs(_("ERROR: No pages found!\n"), stderr);
   else
-    fputs("INFO: Ready to print.\n", stderr);
+    fputs(_("INFO: Ready to print.\n"), stderr);
 
   return (Page == 0);
 }
 
 
 /*
- * End of "$Id: rastertolabel.c 5703 2006-06-29 18:12:04Z mike $".
+ * End of "$Id: rastertolabel.c 6820 2007-08-20 21:15:28Z mike $".
  */
