@@ -1,25 +1,16 @@
 /*
- * "$Id: job.h 5393 2006-04-14 18:17:18Z mike $"
+ * "$Id: job.h 6755 2007-08-01 19:02:47Z mike $"
  *
  *   Print job definitions for the Common UNIX Printing System (CUPS) scheduler.
  *
- *   Copyright 1997-2006 by Easy Software Products, all rights reserved.
+ *   Copyright 2007 by Apple Inc.
+ *   Copyright 1997-2007 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "LICENSE.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ *   which should have been included with this file.  If this file is
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  */
 
 /*
@@ -31,6 +22,7 @@ typedef struct cupsd_job_s
   int			id,		/* Job ID */
 			priority;	/* Job priority */
   ipp_jstate_t		state_value;	/* Cached job-state */
+  int			pending_timeout;/* Non-zero if the job was created and waiting on files */
   char			*username;	/* Printing user */
   char			*dest;		/* Destination printer or class */
   cups_ptype_t		dtype;		/* Destination type (class/remote bits) */
@@ -42,12 +34,18 @@ typedef struct cupsd_job_s
   time_t		hold_until;	/* Hold expiration date/time */
   ipp_attribute_t	*state;		/* Job state */
   ipp_attribute_t	*job_sheets;	/* Job sheets (NULL if none) */
+  ipp_attribute_t	*printer_message,
+					/* job-printer-state-message */
+			*printer_reasons;
+					/* job-printer-state-reasons */
   int			current_file;	/* Current file in job */
   ipp_t			*attrs;		/* Job attributes */
   int			print_pipes[2],	/* Print data pipes */
 			back_pipes[2],	/* Backchannel pipes */
+			side_pipes[2],	/* Sidechannel pipes */
 			status_pipes[2];/* Status pipes */
   cupsd_statbuf_t	*status_buffer;	/* Status buffer for this job */
+  int			status_level;	/* Highest log level in a status message */
   int			cost;		/* Filtering cost */
   int			filters[MAX_FILTERS + 1];
 					/* Filter process IDs, 0 terminated */
@@ -55,6 +53,13 @@ typedef struct cupsd_job_s
   int			status;		/* Status code from filters */
   cupsd_printer_t	*printer;	/* Printer this job is assigned to */
   int			tries;		/* Number of tries for this job */
+  char			*auth_username,	/* AUTH_USERNAME environment variable, if any */
+			*auth_domain,	/* AUTH_DOMAIN environment variable, if any */
+			*auth_password;	/* AUTH_PASSWORD environment variable, if any */
+#ifdef HAVE_GSSAPI
+  krb5_ccache		ccache;		/* Kerberos credential cache */
+  char			*ccname;	/* KRB5CCNAME environment variable */
+#endif /* HAVE_GSSAPI */
 } cupsd_job_t;
 
 
@@ -93,7 +98,8 @@ VAR int			JobRetryLimit	VALUE(5),
  */
 
 extern cupsd_job_t	*cupsdAddJob(int priority, const char *dest);
-extern void		cupsdCancelJob(cupsd_job_t *job, int purge);
+extern void		cupsdCancelJob(cupsd_job_t *job, int purge,
+			               ipp_jstate_t newstate);
 extern void		cupsdCancelJobs(const char *dest, const char *username,
 			                int purge);
 extern void		cupsdCheckJobs(void);
@@ -101,7 +107,6 @@ extern void		cupsdCleanJobs(void);
 extern void		cupsdDeleteJob(cupsd_job_t *job);
 extern cupsd_job_t	*cupsdFindJob(int id);
 extern void		cupsdFinishJob(cupsd_job_t *job);
-extern void		cupsdUnloadCompletedJobs(void);
 extern void		cupsdFreeAllJobs(void);
 extern int		cupsdGetPrinterJobCount(const char *dest);
 extern int		cupsdGetUserJobCount(const char *username);
@@ -115,11 +120,12 @@ extern void		cupsdSaveAllJobs(void);
 extern void		cupsdSaveJob(cupsd_job_t *job);
 extern void		cupsdSetJobHoldUntil(cupsd_job_t *job, const char *when);
 extern void		cupsdSetJobPriority(cupsd_job_t *job, int priority);
-extern void		cupsdStopAllJobs(void);
+extern void		cupsdStopAllJobs(int force);
 extern void		cupsdStopJob(cupsd_job_t *job, int force);
-extern void		cupsdUpdateJob(cupsd_job_t *job);
+extern void		cupsdTimeoutJob(cupsd_job_t *job);
+extern void		cupsdUnloadCompletedJobs(void);
 
 
 /*
- * End of "$Id: job.h 5393 2006-04-14 18:17:18Z mike $".
+ * End of "$Id: job.h 6755 2007-08-01 19:02:47Z mike $".
  */
