@@ -1,25 +1,16 @@
 /*
- * "$Id: subscriptions.c 5991 2006-09-29 02:26:29Z mike $"
+ * "$Id: subscriptions.c 6649 2007-07-11 21:46:42Z mike $"
  *
  *   Subscription routines for the Common UNIX Printing System (CUPS) scheduler.
  *
- *   Copyright 1997-2006 by Easy Software Products, all rights reserved.
+ *   Copyright 2007 by Apple Inc.
+ *   Copyright 1997-2007 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "LICENSE.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ *   which should have been included with this file.  If this file is
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
  * Contents:
  *
@@ -34,13 +25,13 @@
  *   cupsdLoadAllSubscriptions()   - Load all subscriptions from the .conf file.
  *   cupsdSaveAllSubscriptions()   - Save all subscriptions to the .conf file.
  *   cupsdStopAllNotifiers()       - Stop all notifier processes.
- *   cupsdUpdateNotifierStatus()   - Read messages from notifiers.
  *   cupsd_compare_subscriptions() - Compare two subscriptions.
  *   cupsd_delete_event()          - Delete a single event...
  *   cupsd_send_dbus()             - Send a DBUS notification...
  *   cupsd_send_notification()     - Send a notification for the specified
  *                                   event.
  *   cupsd_start_notifier()        - Start a notifier subprocess...
+ *   cupsd_update_notifier()       - Read messages from notifiers.
  */
 
 /*
@@ -52,8 +43,8 @@
 #  include <dbus/dbus.h>
 #  ifdef HAVE_DBUS_MESSAGE_ITER_INIT_APPEND
 #    define dbus_message_append_iter_init dbus_message_iter_init_append
-#    define dbus_message_iter_append_string(i,v) dbus_message_iter_append_basic(i, DBUS_TYPE_STRING, v)
-#    define dbus_message_iter_append_uint32(i,v) dbus_message_iter_append_basic(i, DBUS_TYPE_UINT32, v)
+#    define dbus_message_iter_append_string(i,v) dbus_message_iter_append_basic(i, DBUS_TYPE_STRING, &(v))
+#    define dbus_message_iter_append_uint32(i,v) dbus_message_iter_append_basic(i, DBUS_TYPE_UINT32, &(v))
 #  endif /* HAVE_DBUS_MESSAGE_ITER_INIT_APPEND */
 #endif /* HAVE_DBUS */
 
@@ -73,6 +64,7 @@ static void	cupsd_send_dbus(cupsd_eventmask_t event, cupsd_printer_t *dest,
 static void	cupsd_send_notification(cupsd_subscription_t *sub,
 		                        cupsd_event_t *event);
 static void	cupsd_start_notifier(cupsd_subscription_t *sub);
+static void	cupsd_update_notifier(void);
 
 
 /*
@@ -1226,10 +1218,7 @@ cupsdStopAllNotifiers(void)
 
   if (NotifierPipes[0] >= 0)
   {
-    cupsdLogMessage(CUPSD_LOG_DEBUG2,
-                    "cupsdStopAllNotifiers: Removing fd %d from InputSet...",
-	            NotifierPipes[0]);
-    FD_CLR(NotifierPipes[0], InputSet);
+    cupsdRemoveSelect(NotifierPipes[0]);
 
     cupsdStatBufDelete(NotifierStatusBuffer);
 
@@ -1243,24 +1232,6 @@ cupsdStopAllNotifiers(void)
 }
 
 
-/*
- * 'cupsdUpdateNotifierStatus()' - Read messages from notifiers.
- */
-
-void
-cupsdUpdateNotifierStatus(void)
-{
-  char		message[1024];		/* Pointer to message text */
-  int		loglevel;		/* Log level for message */
-
-
-  while (cupsdStatBufUpdate(NotifierStatusBuffer, &loglevel,
-                            message, sizeof(message)))
-    if (!strchr(NotifierStatusBuffer->buffer, '\n'))
-      break;
-}
-
-
 /*
  * 'cupsd_compare_subscriptions()' - Compare two subscriptions.
  */
@@ -1362,11 +1333,11 @@ cupsd_send_dbus(cupsd_eventmask_t event,/* I - Event to send */
 
   dbus_message_append_iter_init(message, &iter);
   if (dest)
-    dbus_message_iter_append_string(&iter, &(dest->name));
+    dbus_message_iter_append_string(&iter, dest->name);
   if (job)
   {
-    dbus_message_iter_append_uint32(&iter, &(job->id));
-    dbus_message_iter_append_string(&iter, &(job->username));
+    dbus_message_iter_append_uint32(&iter, job->id);
+    dbus_message_iter_append_string(&iter, job->username);
   }
 
   dbus_connection_send(con, message, NULL);
@@ -1571,11 +1542,8 @@ cupsd_start_notifier(
 
     NotifierStatusBuffer = cupsdStatBufNew(NotifierPipes[0], "[Notifier]");
 
-    cupsdLogMessage(CUPSD_LOG_DEBUG2,
-                    "start_notifier: Adding fd %d to InputSet...",
-		    NotifierPipes[0]);
-
-    FD_SET(NotifierPipes[0], InputSet);
+    cupsdAddSelect(NotifierPipes[0], (cupsd_selfunc_t)cupsd_update_notifier,
+                   NULL, NULL);
   }
 
   if (cupsdOpenPipe(fds))
@@ -1597,7 +1565,7 @@ cupsd_start_notifier(
   */
 
   if (cupsdStartProcess(command, argv, envp, fds[0], -1, NotifierPipes[1],
-			-1, 0, &pid) < 0)
+			-1, -1, 0, &pid) < 0)
   {
    /*
     * Error - can't fork!
@@ -1627,5 +1595,23 @@ cupsd_start_notifier(
 
 
 /*
- * End of "$Id: subscriptions.c 5991 2006-09-29 02:26:29Z mike $".
+ * 'cupsd_update_notifier()' - Read messages from notifiers.
+ */
+
+void
+cupsd_update_notifier(void)
+{
+  char		message[1024];		/* Pointer to message text */
+  int		loglevel;		/* Log level for message */
+
+
+  while (cupsdStatBufUpdate(NotifierStatusBuffer, &loglevel,
+                            message, sizeof(message)))
+    if (!strchr(NotifierStatusBuffer->buffer, '\n'))
+      break;
+}
+
+
+/*
+ * End of "$Id: subscriptions.c 6649 2007-07-11 21:46:42Z mike $".
  */
